/// @addtogroup struct
/// @{

/////////////////////////////////////////////////////////////////////////////
///
/// @file digraph.h
///
/// This header file contains the definition of a weighted directed graph
/// class and several algorithms on this graph, including some minimal path
/// algorithms with rounding control to compute rigorous results.
///
/// @author Pawel Pilarczyk
///
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 1997-2009 by Pawel Pilarczyk.
//
// This file is part of the Homology Library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

// Started in January 2006. Last revision: June 15, 2009.


#ifndef _CHOMP_STRUCT_DIGRAPH_H_
#define _CHOMP_STRUCT_DIGRAPH_H_

#include <iostream>
#include <new>
#include <stack>
#include <set>
#include <memory>
#include <vector>

#include "chomp/struct/multitab.h"
#include "chomp/struct/flatmatr.h"
#include "chomp/struct/bitfield.h"
#include "chomp/struct/bitsets.h"
#include "chomp/struct/fibheap.h"
#include "chomp/system/textfile.h"
#include "chomp/system/timeused.h"

namespace chomp {
namespace homology {


// --------------------------------------------------
// ----------------- DUMMY ROUNDING -----------------
// --------------------------------------------------

/// A dummy class for rounding operations which does not actually do
/// any rounding. Please, use it as a template for your own rounding classes.
template <class numType>
class dummyRounding
{
public:
	/// Adds two numbers with the result rounded downwards.
	static inline numType add_down (const numType &x, const numType &y)
	{ return x + y; }

	/// Adds two numbers with the result rounded upwards.
	static inline numType add_up (const numType &x, const numType &y)
	{ return x + y; }

	/// Subtracts two numbers with the result rounded downwards.
	static inline numType sub_down (const numType &x, const numType &y)
	{ return x - y; }

	/// Subtracts two numbers with the result rounded upwards.
	static inline numType sub_up (const numType &x, const numType &y)
	{ return x - y; }

	/// Multiplies two numbers with the result rounded downwards.
	static inline numType mul_down (const numType &x, const numType &y)
	{ return x * y; }

	/// Multiplies two numbers with the result rounded upwards.
	static inline numType mul_up (const numType &x, const numType &y)
	{ return x * y; }

	/// Divides two numbers with the result rounded downwards.
	static inline numType div_down (const numType &x, const numType &y)
	{ return x / y; }

	/// Divides a number by an integer with the result rounded downwards.
	static inline numType div_down (const numType &x, int y)
	{ return x / y; }

	/// Divides two numbers with the result rounded upwards.
	static inline numType div_up (const numType &x, const numType &y)
	{ return x / y; }

private:
}; /* class dummyRounding */


// --------------------------------------------------
// ------------------ DUMMY ARRAY -------------------
// --------------------------------------------------

/// A dummy array of integers that ignores all the assigned values.
class dummyArray
{
public:
	/// The constructor of a dummy array.
	dummyArray () {n = 0;}

	/// Operator [] which returns a reference to a dummy value.
	int &operator [] (int) {return n;}

private:
	/// A dummy number used as a dump box for assigning values.
	int n;

}; /* class dummyArray */


// --------------------------------------------------
// ----------------- DIRECTED GRAPH -----------------
// --------------------------------------------------

// #define DIGRAPH_DEBUG

/// This class defines a directed graph with very limited number of
/// operations, and a few specific algorithms implemented on it, like DFS.
/// The graph can be treated as weighted if necessary.
template <class wType = int>
class diGraph
{
public:
	/// The type of the weight of edges.
	typedef wType weight_type;
	
	/// The default constructor of an empty graph.
	/// Note: The default copy constructor and assignment operator
	/// generated by the compiler are good for copying graphs.
	diGraph ();

	/// The destructor.
	~diGraph ();

	/// Swaps the data with another graph.
	void swap (diGraph<wType> &g);

	/// Adds a vertex.
	void addVertex (void);

	/// Adds an edge starting at the last vertex.
	/// Note: The range of the target vertex number is not verified.
	void addEdge (int target);

	/// Adds an edge from the last vertex to the given one
	/// and sets the weight of this edge.
	void addEdge (int target, const wType &weight);

	/// Sets the weight of the given edge.
	void setWeight (int vertex, int i, const wType &weight);

	/// Sets the weight of the given edge.
	void setWeight (int edge, const wType &weight);
	
	/// Sets the weights of all the edges at a time.
	/// The weights are pulled from the table with the [] operator.
	template <class Table>
	void setWeights (const Table &tab);

	/// Removes the last vertex and all the edges going out from it.
	/// This is done in the time O(1).
	void removeVertex (void);

	/// Removes the given vertex and all the edges going out from it,
	/// as well as the edges going towards it.
	/// If requested, the weights in the graph are also updated.
	/// This function might be slow - it is done in the time O(V+E).
	void removeVertex (int vertex, bool updateweights = false);

	/// Returns the number of vertices.
	int countVertices (void) const;

	/// Returns the number of edges.
	int countEdges (void) const;

	/// Counts the number of edges leaving the given vertex.
	int countEdges (int vertex) const;
	
	/// Retrieves the given edge that leaves the given vertex.
	int getEdge (int vertex, int i) const;

	/// Retrieves the weight of the given edge.
	const wType &getWeight (int vertex, int i) const;

	/// Retrieves the weight of the given edge.
	const wType &getWeight (int edge) const;

	/// Gets the weights of all the edges at a time.
	/// The weights are put into the table with the [] operator.
	template <class Table>
	void getWeights (Table &tab) const;

	/// Fills out a table that represents all the edges of the graph.
	void writeEdges (int (*table) [2]) const;

	/// Creates a transposed graph.
	template <class wType1>
	void transpose (diGraph<wType1> &result,
		bool copyweights = false) const;

	/// Computes a restriction of the graph to its subgraph. The subgraph
	/// vertices are defined by nonzero entries in the supplied table.
	/// The result must be initially empty.
	template <class Table, class wType1>
	void subgraph (diGraph<wType1> &result, const Table &tab,
		bool copyweights = false) const;

	/// Marks each vertex visited by DFS with the given color,
	/// starting with the given vertex. Runs for one component only.
	/// The initial color in 'tab' must be different than the given one.
	template <class Table, class Color>
	void DFScolor (Table &tab, const Color &color, int vertex = 0) const;

	/// The recurrent procedure for DFScolor.
	template <class Table, class Color>
	void DFScolorRecurrent (Table &tab, const Color &color,
		int vertex = 0) const;

	/// A stack version of the recurrent procedure for DFScolor.
	template <class Table, class Color>
	void DFScolorStack (Table &tab, const Color &color,
		int vertex = 0) const;

	/// Computes the DFS finishing time for each vertex.
	/// Note: The time begins with 1, not with 0.
	template <class Table>
	void DFSfinishTime (Table &tab) const;

	/// Computes the DFS forest.
	/// Considers the vertices in the given order.
	/// Saves the numbers of vertices of each tree in 'compVertices',
	/// and keeps the one-beyond-the-end offsets of the trees in the
	/// table 'compEnds'. Records the connections between the trees
	/// in 'scc' (which must be empty when this function is called).
	/// If requested, only those single-vertex trees are counted
	/// which have an edge that loops back to themselves.
	/// Returns the number of trees in the computed forest.
	template <class Table1, class Table2, class Table3>
	int DFSforest (const Table1 &ordered, Table2 &compVertices,
		Table3 &compEnds, bool nontrivial = false,
		diGraph<wType> *sccGraph = 0) const;

	/// Computes the length of the shortest path from the given vertex
	/// to another one. The length is measured by counting edges.
	/// Uses a stack version of the BFS algorithm.
	/// Returns the length of the path, 0 or -1 if none.
	int shortestPath (int source, int destination) const;

	/// Computes the length of the shortest loop from the given vertex
	/// to itself. The length is measured by counting edges on the way.
	/// Uses a stack version of the BFS algorithm.
	/// Returns the length of the loop or 0 if none.
	int shortestLoop (int origin) const;

	/// Dijkstra's algorithm for solving the single-source shortest
	/// paths problem if all the edge weights are nonnegative.
	/// The table 'len' is used to store the path lengths during the
	/// computations and contains the final result. A negative value
	/// stands for the infinity (no path to the given vertex).
	/// This is a special version that uses the given edge weights
	/// instead of the weights contained in the definition of the graph.
	template <class lenTable, class weightsType, class roundType>
	void Dijkstra (const roundType &rounding, int source, lenTable &len,
		weightsType &edgeWeights) const;

	/// Dijkstra's algorithm running on the graph's own weights.
	template <class lenTable, class roundType>
	void Dijkstra (const roundType &rounding, int source, lenTable &len)
		const;

	/// The above algorithm without rounding control.
	template <class lenTable>
	void Dijkstra (int source, lenTable &len) const;

	/// Runs the Bellman-Ford algorithm which computes the single-source
	/// shortest paths in a weighted directed graph, where some edge
	/// weights may be negative. Runs in O(V*E).
	/// The table 'len' is used to store the path lengths during the
	/// computations and contains the final result. The number for
	/// infinity is set to indicate unreachable vertices.
	/// The table with predecessors allows to retrieve shortest paths;
	/// this must be a pointer to an array-like structure, e.g., int **.
	/// To ignore this data, use an object of the 'dummyArray' class.
	/// Returns true if successful, false if a negative cycle is found.
	template <class lenTable, class predTable, class roundType>
	bool BellmanFord (const roundType &rounding, int source,
		lenTable &len, wType *infinity, predTable pred) const;

	/// The above algorithm without rounding control.
	template <class lenTable, class predTable>
	bool BellmanFord (int source, lenTable &len, wType *infinity,
		predTable pred) const;

	/// Runs the Bellman-Ford algorithm (see above) without storing the
	/// distances, only returns the information about the existence
	/// of a negative-weight cycle.
	template <class roundType>
	bool BellmanFord (const roundType &rounding, int source) const;

	/// The above algorithm without rounding control.
	bool BellmanFord (int source) const;

	/// Runs the Edmonds algorithm to compute the shortest path
	/// that runs through all the vertices of the graph.
	/// Computation time: O (n log n). The length of the path
	/// is measured as the sum of the weights of the edges.
	/// The path does not contain any loops.
	/// The graph should be strongly connected.
	wType Edmonds () const;

	/// An old implementation of the Edmonds algorithm (less efficient).
	wType EdmondsOld () const;

	/// Runs the Floyd-Warshall algorithm to compute the shortest
	/// paths between all pairs of vertices in the graph.
	/// The position [i] [j] of the array contains
	/// the length of the shortest path from vertex i to vertex j.
	/// Provides a rigorous lower bound in interval arithmetic,
	/// provided that intervals are compared with "<" and "<="
	/// by comparing their lower ends only.
	/// If "setInfinity" is "true", then computes a value that serves
	/// as the infinity, fills in the corresponding entries in "arr",
	/// and returns this value.
	/// Otherwise, returns the length of the shortest path. In this
	/// case, arr [i] [j] is undefined if there is no path i -> j.
	/// Throws an error message if a negative loop is found,
	/// unless "ignoreNegLoop" is set to "true".
	template <class arrayType, class roundType>
	wType FloydWarshall (const roundType &rounding, arrayType &arr,
		bool setInfinity = true, bool ignoreNegLoop = false) const;

	/// The above algorithm without rounding control.
	template <class arrayType>
	wType FloydWarshall (arrayType &arr, bool setInfinity = true,
		bool ignoreNegLoop = false) const;

	/// Runs Johnson's algorithm to compute the minimum path weight
	/// between any vertices in the graph. The time complexity of this
	/// algorithm is essentially O (V^2 log V + VE log V),
	/// which is better than the complexity of the Warshall-Floyd
	/// algorithm for sparse graphs, that is, graphs in which the number
	/// of edges E is of order smaller than V^2.
	/// The meaning of the arguments and the returned value is the same
	/// as in 'FloydWarshall'.
	template <class arrayType, class roundType>
	wType Johnson (const roundType &rounding, arrayType &arr,
		bool setInfinity = true, bool ignoreNegLoop = false) const;

	/// The above algorithm without rounding control.
	template <class arrayType>
	wType Johnson (arrayType &arr, bool setInfinity = true,
		bool ignoreNegLoop = false) const;

	/// Uses the Floyd-Warshall algorithm or Johnson's algorithm,
	/// depending on the number of edges, to compute the minimum
	/// path weight between any vertices in the graph.
	/// Throws an error message if a negative loop exists in the graph,
	/// unless "ignoreNegLoop" is set to "true".
	/// To force the use of Johnson's algorithm, set "sparseGraph" to 1,
	/// to force the use of Warshall-Floyd, set "sparseGraph" to 0,
	/// otherwise it will be determined heuristically which algorithm
	/// should be used.
	template <class roundType>
	wType minPathWeight (const roundType &rounding,
		bool ignoreNegLoop = false, int sparseGraph = -1) const;

	/// The above algorithm without rounding control.
	wType minPathWeight (bool ignoreNegLoop = false,
		int sparseGraph = -1) const;

	/// Runs the Karp algorithm for each strongly connected component
	/// of the graph and returns the minimum mean cycle weight,
	/// which can be negative.
	/// As a byproduct, saves the transposed graph, if requested to.
	wType minMeanCycleWeight (diGraph<wType> *transposed = 0) const;

	/// A version of the above function modified for the purpose
	/// of interval arithmetic to provide the correct lower bound
	/// for the minimum mean cycle weight in a graph.
	/// This specialization is necessary, because of the subtraction
	/// operation used in Karp's algorithm. Therefore, upper and lower
	/// bounds for the minimum path progression weights must be computed
	/// independently.
	/// The intervals are compared by comparing their lower bounds only.
	template <class roundType>
	wType minMeanCycleWeight (const roundType &rounding,
		diGraph<wType> *transposed) const;

	/// Runs an algorithm based on Karp's idea to compute the minimum
	/// mean path weight for paths starting at any of the given
	/// vertices and of length not exceeding the number of vertices
	/// in the graph.
	/// Returns 0 if no path starts at any of the vertices.
	template <class arrayType, class roundType>
	wType minMeanPathWeight (const roundType &rounding,
		const arrayType &starting, int n) const;

	/// The above algorithm without rounding control.
	template <class arrayType>
	wType minMeanPathWeight (const arrayType &starting, int n) const;

	/// Operator == for comparing digraphs. No isomorphism check, just
	/// comparing with the same order of vertices. Ignores weights.
	template <class wType1, class wType2>
	friend bool operator == (const diGraph<wType1> &g1,
		const diGraph<wType2> &g2);

	/// Outputs the graph to a text stream in a human-readable format.
	template <class outType>
	outType &show (outType &out, bool showWeights = false) const;

protected:
	/// The number of vertices.
	int nVertices;

	/// A table with the offsets of the one-after-the-last edge
	/// of each vertex.
	multitable<int> edgeEnds;

	/// A table with edge target numbers.
	multitable<int> edges;

	/// A table with edge weights.
	multitable<wType> weights;

private:
	/// The recurrent procedure for DFSfinishTime.
	template <class Table>
	void DFSfinishTimeRecurrent (Table &tab, int vertex,
		int &counter) const;

	/// A stack version of the recurrent procedure for DFSfinishTime.
	template <class Table>
	void DFSfinishTimeStack (Table &tab, int vertex,
		int &counter) const;

	/// The recurrent procedure for DFSforest.
	/// Returns true iff there is a loop within the tree found.
	template <class Table1, class Table2>
	bool DFSforestRecurrent (Table1 &tab, Table1 &ntab, int vertex,
		int treeNumber, int countTrees, Table2 &compVertices,
		int &curVertex, diGraph *sccGraph, int *sccEdgeAdded)
		const;

	/// A stack version of the recurrent procedure for DFSforest.
	template <class Table1, class Table2>
	bool DFSforestStack (Table1 &tab, Table1 &ntab, int vertex,
		int treeNumber, int countTrees, Table2 &compVertices,
		int &curVertex, diGraph *sccGraph, int *sccEdgeAdded)
		const;

	/// An edge with a weight (used by the Edmonds algorithm).
	struct edgeTriple
	{
		/// The starting and ending vertices of the edge.
		int vert1, vert2;
		
		/// The weight of the edge.
		wType weight;

		/// The < operator for comparing the weights of edges.
		friend bool operator < (const edgeTriple &x,
			const edgeTriple &y)
		{
			return (x. weight < y. weight);
		}
	};

	/// A class for representing a positive number with negative values
	/// serving as the infinity (used in the Dijkstra algorithm).
	class posWeight
	{
	public:
		/// The default constructor.
		posWeight ()
		{
			value = 0;
			return;
		}

		/// An optional constructor.
		explicit posWeight (const wType &_value)
		{
			if (_value < 0)
				value = 0;
			else
				value = _value;
			return;
		}

		/// Sets the value to the infinity.
		void setInfinity ()
		{
			value = -1;
			return;
		}

		/// Returns true iff the value corresponds to the infinity.
		bool isInfinity () const
		{
			return (value < 0);
		}

		/// Returns the value stored in this object.
		const wType &getValue () const
		{
			return value;
		}

		/// The < operator for comparing the numbers.
		friend bool operator < (const posWeight &x,
			const posWeight &y)
		{
			if (y. isInfinity ())
				return !(x. isInfinity ());
			else if (x. isInfinity ())
				return false;
			return (x. value < y. value);
		}

		/// The operator for showing the number to the output stream.
		friend std::ostream &operator << (std::ostream &out,
			const posWeight &x)
		{
			if (x. isInfinity ())
				out << "+oo";
			else
				out << x. getValue ();
			return out;
		}

	private:
		/// The actual number.
		wType value;
	};

}; /* class diGraph */

// --------------------------------------------------

template <class wType>
inline diGraph<wType>::diGraph (): nVertices (0),
	edgeEnds (1024), edges (4096), weights (4096)
{
	return;
} /* diGraph::diGraph */

template <class wType>
inline diGraph<wType>::~diGraph ()
{
	return;
} /* diGraph::~diGraph */

// --------------------------------------------------

template <class wType1, class wType2>
inline bool operator == (const diGraph<wType1> &g1,
	const diGraph<wType2> &g2)
{
	if (g1. nVertices != g2. nVertices)
		return false;
	if (!g1. nVertices)
		return true;
	for (int i = 0; i < g1. nVertices; ++ i)
		if (g1. edgeEnds [i] != g2. edgeEnds [i])
			return false;
	int nEdges = g1. edgeEnds [g1. nVertices - 1];
	for (int i = 0; i < nEdges; ++ i)
		if (g1. edges [i] != g2. edges [i])
			return false;
	return true;
} /* operator == */

template <class wType1, class wType2>
inline bool operator != (const diGraph<wType1> &g1,
	const diGraph<wType2> &g2)
{
	return !(g1 == g2);
} /* operator != */

// --------------------------------------------------

template <class wType>
inline void diGraph<wType>::swap (diGraph<wType> &g)
{
	my_swap (nVertices, g. nVertices);
	edgeEnds. swap (g. edgeEnds);
	edges. swap (g. edges);
	weights. swap (g. weights);
	return;
} /* diGraph::swap */

template <class wType>
inline void diGraph<wType>::addVertex (void)
{
	edgeEnds [nVertices] = nVertices ? edgeEnds [nVertices - 1] : 0;
	++ nVertices;
	return;
} /* diGraph::addVertex */

template <class wType>
inline void diGraph<wType>::addEdge (int target)
{
	if (!nVertices)
		throw "Trying to add an edge to an empty graph.";
//	if (target >= nVertices)
//		throw "Trying to add an edge to a nonexistent vertex.";
	if (target < 0)
		throw "Trying to add an edge to a negative vertex.";
	int &offset = edgeEnds [nVertices - 1];
	if (offset + 1 <= 0)
		throw "Too many edges in a diGraph (limit = 2,147,483,647).";
	edges [offset] = target;
	++ offset;
	return;
} /* diGraph::addEdge */

template <class wType>
inline void diGraph<wType>::addEdge (int target, const wType &weight)
{
	if (!nVertices)
		throw "Trying to add an edge to an empty graph.";
//	if (target >= nVertices)
//		throw "Trying to add an edge to a nonexistent vertex.";
	if (target < 0)
		throw "Trying to add an edge to a negative vertex.";
	int &offset = edgeEnds [nVertices - 1];
	if (offset + 1 <= 0)
		throw "Too many edges in a diGraph (limit = 2,147,483,647).";
	edges [offset] = target;
	weights [offset] = weight;
	++ offset;
	return;
} /* diGraph::addEdge */

template <class wType>
inline void diGraph<wType>::setWeight (int vertex, int i,
	const wType &weight)
{
	if (!vertex)
		weights [i] = weight;
	else
		weights [edgeEnds [vertex - 1] + i] = weight;
	return;
} /* diGraph::setWeight */

template <class wType>
inline void diGraph<wType>::setWeight (int edge, const wType &weight)
{
	weights [edge] = weight;
	return;
} /* diGraph::setWeight */

template <class wType> template <class Table>
inline void diGraph<wType>::setWeights (const Table &tab)
{
	if (!nVertices)
		return;
	int nEdges = edgeEnds [nVertices - 1];
	for (int i = 0; i < nEdges; ++ i)
		weights [i] = tab [i];
	return;
} /* diGraph::setWeights */

template <class wType>
inline void diGraph<wType>::removeVertex (void)
{
	if (!nVertices)
		throw "Trying to remove a vertex from the empty graph.";
	-- nVertices;
	return;
} /* diGraph::removeVertex */

template <class wType>
inline void diGraph<wType>::removeVertex (int vertex, bool updateweights)
{
	// make sure that the vertex number is within the scope
	if ((vertex < 0) || (vertex >= nVertices))
		throw "Trying to remove a vertex that is not in the graph.";

	// remove edges that begin or end at the given vertex
	int curEdge = 0;
	int newEdge = 0;
	int skipped = 0;
	for (int v = 0; v < nVertices; ++ v)
	{
		// skip the edges that begin at the given vertex
		if (!skipped && (v == vertex))
		{
			curEdge = edgeEnds [v];
			++ skipped;
			continue;
		}

		// skip the edges that point to the given vertex
		int maxEdge = edgeEnds [v];
		for (; curEdge < maxEdge; ++ curEdge)
		{
			if (edges [curEdge] == vertex)
				continue;
			int target = edges [curEdge];
			edges [newEdge] = (target < vertex) ? target :
				(target - 1);
			if (updateweights)
				weights [newEdge] = weights [curEdge];
			++ newEdge;
		}
		edgeEnds [v - skipped] = newEdge;
	}

	// decrease the number of vertices
	nVertices -= skipped;

	return;
} /* diGraph::removeVertex */

template <class wType>
inline int diGraph<wType>::countVertices (void) const
{
	return nVertices;
} /* diGraph::countVertices */

template <class wType>
inline int diGraph<wType>::countEdges (void) const
{
	if (!nVertices)
		return 0;
	else
		return edgeEnds [nVertices - 1];
} /* diGraph::countEdges */

template <class wType>
inline int diGraph<wType>::countEdges (int vertex) const
{
	if (!vertex)
		return edgeEnds [0];
	else
		return edgeEnds [vertex] - edgeEnds [vertex - 1];
} /* diGraph::countEdges */

template <class wType>
inline int diGraph<wType>::getEdge (int vertex, int i) const
{
	if (!vertex)
		return edges [i];
	else
		return edges [edgeEnds [vertex - 1] + i];
} /* diGraph::getEdge */

template <class wType>
inline const wType &diGraph<wType>::getWeight (int vertex, int i) const
{
	if (!vertex)
		return weights [i];
	else
		return weights [edgeEnds [vertex - 1] + i];
} /* diGraph::getWeight */

template <class wType>
inline const wType &diGraph<wType>::getWeight (int edge) const
{
	return weights [edge];
} /* diGraph::getWeight */

template <class wType> template <class Table>
inline void diGraph<wType>::getWeights (Table &tab) const
{
	if (!nVertices)
		return;
	int nEdges = edgeEnds [nVertices - 1];
	for (int i = 0; i < nEdges; ++ i)
		tab [i] = weights [i];
	return;
} /* diGraph::getWeights */

template <class wType>
inline void diGraph<wType>::writeEdges (int (*table) [2]) const
{
	int curEdge = 0;
	for (int curVertex = 0; curVertex < nVertices; ++ curVertex)
	{
		int maxEdge = edgeEnds [curVertex];
		while (curEdge < maxEdge)
		{
			table [curEdge] [0] = curVertex;
			table [curEdge] [1] = edges [curEdge];
			++ curEdge;
		}
	}
	return;
} /* diGraph::writeEdges */

template <class wType> template <class wType1>
inline void diGraph<wType>::transpose (diGraph<wType1> &result,
	bool copyweights) const
{
	// prepare the resulting graph
	result. nVertices = nVertices;
	if (!nVertices)
		return;

	// compute the initial offsets for the edge numbers
	for (int i = 0; i < nVertices; ++ i)
		result. edgeEnds [i] = 0;
	int nEdges = edgeEnds [nVertices - 1];
	for (int i = 0; i < nEdges; ++ i)
		if (edges [i] < nVertices - 1)
			++ result. edgeEnds [edges [i] + 1];
	for (int i = 2; i < nVertices; ++ i)
		result. edgeEnds [i] += result. edgeEnds [i - 1];

	// compute the reversed edges
	int curEdge = 0;
	for (int i = 0; i < nVertices; ++ i)
	{
		for (; curEdge < edgeEnds [i]; ++ curEdge)
		{
			int j = edges [curEdge];
			int &offset = result. edgeEnds [j];
			result. edges [offset] = i;
			if (copyweights)
				result. weights [offset] = weights [curEdge];
			++ offset;
		}
	}
	return;
} /* diGraph::transpose */

template <class wType> template <class Table, class wType1>
inline void diGraph<wType>::subgraph (diGraph<wType1> &g,
	const Table &tab, bool copyweights) const
{
	// compute the new numbers of vertices that remain in the graph
	int *numbers = new int [nVertices];
	int curNumber = 0;
	for (int i = 0; i < nVertices; ++ i)
		if (tab [i])
			numbers [i] = curNumber ++;
		else
			numbers [i] = -1;

	// copy the edges from the old graph to the new one
	for (int i = 0; i < nVertices; ++ i)
	{
		if (numbers [i] < 0)
			continue;
		g. addVertex ();
		int firstEdge = i ? edgeEnds [i - 1] : 0;
		int endEdge = edgeEnds [i];
		for (int j = firstEdge; j < endEdge; ++ j)
		{
			int edgeEnd = edges [j];
			if (numbers [edgeEnd] >= 0)
			{
				if (copyweights)
					g. addEdge (numbers [edgeEnd],
						weights [j]);
				else
					g. addEdge (numbers [edgeEnd]);
			}
		}
	}

	// clean up memory and exit
	delete [] numbers;
	return;
} /* diGraph::subgraph */

// --------------------------------------------------

template <class wType> template <class Table, class Color>
inline void diGraph<wType>::DFScolorRecurrent (Table &tab,
	const Color &color, int vertex) const
{
	tab [vertex] = color;
	int maxEdge = edgeEnds [vertex];
	for (int i = vertex ? edgeEnds [vertex - 1] : 0; i < maxEdge; ++ i)
	{
		int next = edges [i];
		if (tab [next] != color)
			DFScolorRecurrent (tab, color, next);
	}
	return;
} /* diGraph::DFScolorRecurrent */

template <class wType> template <class Table, class Color>
inline void diGraph<wType>::DFScolorStack (Table &tab, const Color &color,
	int vertex) const
{
	// prepare stacks for the recursion
	std::stack<int> s_vertex;
	std::stack<int> s_edge;
	std::stack<int> s_maxedge;

	// mark the current vertex as visited
	tab [vertex] = color;

	// determine the edges to be visited
	int edge = vertex ? edgeEnds [vertex - 1] : 0;
	int maxedge = edgeEnds [vertex];

	while (1)
	{
		// return to the previous recursion level
		// if all the edges have been followed
		if (edge >= maxedge)
		{
			// return if this is the initial recursion level
			if (s_vertex. empty ())
				return;

			// restore the variables from the previous level
			vertex = s_vertex. top ();
			s_vertex. pop ();
			edge = s_edge. top ();
			s_edge. pop ();
			maxedge = s_maxedge. top ();
			s_maxedge. pop ();
			continue;
		}

		// go to the deeper recursion level if possible
		int next = edges [edge ++];
		if (tab [next] != color)
		{
			// store the previous variables at the stacks
			s_vertex. push (vertex);
			s_edge. push (edge);
			s_maxedge. push (maxedge);

			// set the new vertex
			vertex = next;
			
			// mark the new vertex as visited
			tab [vertex] = color;
			
			// determine the edges to be visited
			edge = vertex ? edgeEnds [vertex - 1] : 0;
			maxedge = edgeEnds [vertex];
		}
	}
} /* diGraph::DFScolorStack */

template <class wType> template <class Table, class Color>
inline void diGraph<wType>::DFScolor (Table &tab, const Color &color,
	int vertex) const
{
	DFScolorStack (tab, color, vertex);
	return;
} /* diGraph::DFScolor */

// --------------------------------------------------

template <class wType> template <class Table>
inline void diGraph<wType>::DFSfinishTimeRecurrent (Table &tab, int vertex,
	int &counter) const
{
	// mark the current vertex as visited
	tab [vertex] = -1;

	// call DFS for the other vertices
	for (int edge = vertex ? edgeEnds [vertex - 1] : 0;
		edge < edgeEnds [vertex]; ++ edge)
	{
		int next = edges [edge];
		if (!tab [next])
			DFSfinishTimeRecurrent (tab, next, counter);
	}

	// record the finishing time for the current vertex and return
	tab [vertex] = ++ counter;
	return;
} /* diGraph::DFSfinishTimeRecurrent */

template <class wType> template <class Table>
inline void diGraph<wType>::DFSfinishTimeStack (Table &tab, int vertex,
	int &counter) const
{
	// prepare stacks for the recursion
	std::stack<int> s_vertex;
	std::stack<int> s_edge;
	std::stack<int> s_maxedge;

	// mark the current vertex as visited
	tab [vertex] = -1;

	// determine the edges to be visited
	int edge = vertex ? edgeEnds [vertex - 1] : 0;
	int maxedge = edgeEnds [vertex];

	while (1)
	{
		// return to the previous recursion level
		// if all the edges have been followed
		if (edge >= maxedge)
		{
			// record the finishing time
			// for the current vertex
			tab [vertex] = ++ counter;

			// return if this is the initial recursion level
			if (s_vertex. empty ())
				return;

			// restore the variables from the previous level
			vertex = s_vertex. top ();
			s_vertex. pop ();
			edge = s_edge. top ();
			s_edge. pop ();
			maxedge = s_maxedge. top ();
			s_maxedge. pop ();
			continue;
		}

		// go to the deeper recursion level if possible
		int next = edges [edge ++];
		if (!tab [next])
		{
			// store the previous variables at the stacks
			s_vertex. push (vertex);
			s_edge. push (edge);
			s_maxedge. push (maxedge);

			// set the new vertex
			vertex = next;
			
			// mark the new vertex as visited
			tab [vertex] = -1;
			
			// determine the edges to be visited
			edge = vertex ? edgeEnds [vertex - 1] : 0;
			maxedge = edgeEnds [vertex];
		}
	}

	return;
} /* diGraph::DFSfinishTimeStack */

template <class wType> template <class Table>
inline void diGraph<wType>::DFSfinishTime (Table &tab) const
{
	// initialize the table and the counter
	for (int i = 0; i < nVertices; ++ i)
		tab [i] = 0;
	int counter = 0;

	// compute the finishing time for each tree in the DFS forest
	for (int i = 0; i < nVertices; ++ i)
		if (!tab [i])
			DFSfinishTimeStack (tab, i, counter);

	#ifdef DIGRAPH_DEBUG
	int *tabdebug = new int [nVertices];
	for (int i = 0; i < nVertices; ++ i)
		tabdebug [i] = 0;
	int counterdebug = 0;
	for (int i = 0; i < nVertices; ++ i)
		if (!tabdebug [i])
			DFSfinishTimeRecurrent (tabdebug, i, counterdebug);
	if (counter != counterdebug)
		throw "DFSfinishTime: Wrong counter.";
	for (int i = 0; i < nVertices; ++ i)
		if (tab [i] != tabdebug [i])
		{
			sbug << "\nDFSfinishTime error: tabRec [" << i <<
				"] = " << tab [i] << ", tabStack [" << i <<
				"] = " << tabdebug [i] << ".\n";
			throw "DFSfinishTime: Wrong numbers.";
		}
	sbug << "DEBUG: DFSfinishTime OK. ";
	#endif // DIGRAPH_DEBUG
	return;
} /* diGraph::DFSfinishTime */

// --------------------------------------------------

template <class wType> template <class Table1, class Table2>
inline bool diGraph<wType>::DFSforestRecurrent (Table1 &tab, Table1 &ntab,
	int vertex, int treeNumber, int countTrees,
	Table2 &compVertices, int &curVertex,
	diGraph<wType> *sccGraph, int *sccEdgeAdded) const
{
	// add the vertex to the tree
	compVertices [curVertex ++] = vertex;

	// mark the vertex as belonging to the current tree
	tab [vertex] = treeNumber;
//	if (sccGraph)
//		ntab [treeNumber - 1] = countTrees;

	// build the tree recursively or record connections
	bool loop = false;
	for (int edge = vertex ? edgeEnds [vertex - 1] : 0;
		edge < edgeEnds [vertex]; ++ edge)
	{
		int next = edges [edge];
		if (!tab [next])
			loop |= DFSforestRecurrent (tab, ntab, next,
				treeNumber, countTrees, compVertices,
				curVertex, sccGraph, sccEdgeAdded);
		else if (tab [next] == treeNumber)
		{
			if (sccGraph)
			{
				int target = ntab [treeNumber - 1];
				if (sccEdgeAdded [target] != treeNumber)
				{
					sccGraph -> addEdge (target);
					sccEdgeAdded [target] = treeNumber;
				}
			}
			loop = true;
		}
		else if (sccGraph)
		{
			int target = ntab [tab [next] - 1];
			if ((target >= 0) &&
				(sccEdgeAdded [target] != treeNumber))
			{
				sccGraph -> addEdge (target);
				sccEdgeAdded [target] = treeNumber;
			}
		}
	}

	return loop;
} /* diGraph::DFSforestRecurrent */

template <class wType> template <class Table1, class Table2>
inline bool diGraph<wType>::DFSforestStack (Table1 &tab, Table1 &ntab,
	int vertex, int treeNumber, int countTrees,
	Table2 &compVertices, int &curVertex,
	diGraph<wType> *sccGraph, int *sccEdgeAdded) const
{
	// prepare stacks for the recursion
	std::stack<int> s_vertex;
	std::stack<int> s_edge;
	std::stack<int> s_maxedge;
	std::stack<bool> s_loop;

	// add the vertex to the tree
	compVertices [curVertex ++] = vertex;

	// mark the vertex as belonging to the current tree
	tab [vertex] = treeNumber;
//	if (sccGraph)
//		ntab [vertex] = countTrees;

	// build the tree recursively or record connections
	bool loop = false;
	int edge = vertex ? edgeEnds [vertex - 1] : 0;
	int maxedge = edgeEnds [vertex];
	while (1)
	{
		// return to the previous recursion level
		// if all the edges have been followed
		if (edge >= maxedge)
		{
			// return if this is the initial recursion level
			if (s_vertex. empty ())
				return loop;

			// restore the variables from the previous level
			vertex = s_vertex. top ();
			s_vertex. pop ();
			edge = s_edge. top ();
			s_edge. pop ();
			maxedge = s_maxedge. top ();
			s_maxedge. pop ();
			loop |= s_loop. top ();
			s_loop. pop ();
			continue;
		}

		// go to the deeper recursion level if possible
		int next = edges [edge ++];
		if (!tab [next])
		{
			// store the previous variables at the stacks
			s_vertex. push (vertex);
			s_edge. push (edge);
			s_maxedge. push (maxedge);
			s_loop. push (loop);

			// set the new vertex
			vertex = next;
			
			// add the vertex to the tree
			compVertices [curVertex ++] = vertex;

			// mark the vertex as belonging to the current tree
			tab [vertex] = treeNumber;

			// determine the edges to be visited
			loop = false;
			edge = vertex ? edgeEnds [vertex - 1] : 0;
			maxedge = edgeEnds [vertex];
		}
		else if (tab [next] == treeNumber)
		{
			if (sccGraph)
			{
				int target = ntab [treeNumber - 1];
				if (sccEdgeAdded [target] != treeNumber)
				{
					sccGraph -> addEdge (target);
					sccEdgeAdded [target] = treeNumber;
				}
			}
			loop = true;
		}
		else if (sccGraph)
		{
			int target = ntab [tab [next] - 1];
			if ((target >= 0) &&
				(sccEdgeAdded [target] != treeNumber))
			{
				sccGraph -> addEdge (target);
				sccEdgeAdded [target] = treeNumber;
			}
		}
	}

	return loop;
} /* diGraph::DFSforestStack */

template <class wType> template <class Table1, class Table2, class Table3>
inline int diGraph<wType>::DFSforest (const Table1 &ordered,
	Table2 &compVertices, Table3 &compEnds, bool nontrivial,
	diGraph<wType> *sccGraph) const
{
	// prepare a table to record the numbers of DFS trees
	// to which the vertices belong (the tree numbers begin with 1)
	int *tab = new int [nVertices];
	for (int i = 0; i < nVertices; ++ i)
		tab [i] = 0;

	// prepare a table to record the numbers of nontrivial trees
	// that correspond to the trees in 'tab' (these numbers begin with 0)
	int *ntab = new int [nVertices];

	// prepare a table to record the numbers of edges already in the
	// scc graph; "sccEdgeAdded [n] = m" indicates that the edge
	// m -> n has been added to the scc graph
	int *sccEdgeAdded = sccGraph ? new int [nVertices] : 0;
	if (sccGraph)
	{
		for (int n = 0; n < nVertices; ++ n)
			sccEdgeAdded [n] = -1;
	}

	// prepare the official DFS tree number
	int treeNumber = 0;

	// prepare the data for keeping the nontrivial trees information
	int countTrees = 0;
	int curVertex = 0;

	// compute the DFS trees and connections between them
	for (int i = 0; i < nVertices; ++ i)
	{
		// take the next vertex
		int vertex = ordered [i];

		// if the vertex already belongs to some tree, skip it
		if (tab [vertex])
			continue;

		// add a vertex corresponding to the component
		if (sccGraph)
			sccGraph -> addVertex ();

		// remember the previous vertex number
		int prevVertex = curVertex;

		// mark the entire component and record connections graph
		if (sccGraph)
			ntab [treeNumber] = countTrees;
		++ treeNumber;
		bool loop = DFSforestStack (tab, ntab, vertex,
			treeNumber, countTrees, compVertices,
			curVertex, sccGraph, sccEdgeAdded);

		// update the index bound for the vertex list
		compEnds [countTrees ++] = curVertex;

		// remove the component if it is trivial
		if (nontrivial && !loop)
		{
			-- countTrees;
			curVertex = prevVertex;
			if (sccGraph)
			{
				ntab [treeNumber - 1] = -1;
				sccGraph -> removeVertex ();
			}
		}
	}

	#ifdef DIGRAPH_DEBUG
	diGraph<wType> *sccGraphdebug = 0;
	if (sccGraph)
		sccGraphdebug = new diGraph<wType>;
	// prepare a table to record the numbers of DFS trees
	// to which the vertices belong (the tree numbers begin with 1)
	int *tabdebug = new int [nVertices];
	for (int i = 0; i < nVertices; ++ i)
		tabdebug [i] = 0;

	// prepare a table to record the numbers of nontrivial trees
	// to which the vertices belong (the tree numbers begin with 0)
	int *ntabdebug = new int [nVertices];

	// prepare a table to record the numbers of vertices from which
	// edges were added to the scc graph
	int *sccEdgeAddeddebug = sccGraph ? new int [nVertices] : 0;
	if (sccGraph)
	{
		for (int n = 0; n < nVertices; ++ n)
			sccEdgeAddeddebug [n] = -1;
	}
	// prepare the official DFS tree number
	int treeNumberdebug = 0;

	// prepare the data for keeping the nontrivial trees information
	int countTreesdebug = 0;
	int curVertexdebug = 0;

	int *compVerticesdebug = new int [nVertices];
	int *compEndsdebug = new int [nVertices];
	
	// compute the DFS trees and connections between them
	for (int i = 0; i < nVertices; ++ i)
	{
		// take the next vertex
		int vertex = ordered [i];

		// if the vertex already belongs to some tree, skip it
		if (tabdebug [vertex])
			continue;

		// add a vertex corresponding to the component
		if (sccGraphdebug)
			sccGraphdebug -> addVertex ();

		// remember the previous vertex number
		int prevVertex = curVertexdebug;

		// mark the entire component and record connections graph
		if (sccGraphdebug)
			ntabdebug [treeNumberdebug] = countTreesdebug;
		++ treeNumberdebug;
		bool loop = DFSforestRecurrent (tabdebug, ntabdebug, vertex,
			treeNumberdebug, countTreesdebug, compVerticesdebug,
			curVertexdebug, sccGraphdebug, sccEdgeAddeddebug);

		// update the index bound for the vertex list
		compEndsdebug [countTreesdebug ++] = curVertexdebug;

		// remove the component if it is trivial
		if (nontrivial && !loop)
		{
			-- countTreesdebug;
			curVertexdebug = prevVertex;
			if (sccGraphdebug)
			{
				ntabdebug [treeNumberdebug - 1] = -1;
				sccGraphdebug -> removeVertex ();
			}
		}
	}
	if (countTrees != countTreesdebug)
		throw "DFSforest: Wrong countTrees.";
	for (int i = 0; i < countTrees; ++ i)
		if (compEnds [i] != compEndsdebug [i])
			throw "DFSforest: Wrong compEnds.";
	for (int i = 0; i < compEndsdebug [countTrees - 1]; ++ i)
		if (compVertices [i] != compVerticesdebug [i])
			throw "DFSforest: Wrong vertices.";
	if (curVertex != curVertexdebug)
		throw "DFSforest: Wrong curVertex.";
	for (int i = 0; i < nVertices; ++ i)
		if (tab [i] != tabdebug [i])
			throw "DFSforest: Wrong tab.";
	if (sccGraph)
	{
		for (int i = 0; i < nVertices; ++ i)
			if (ntab [i] != ntabdebug [i])
				throw "DFSforest: Wrong ntab.";
		if (*sccGraph != *sccGraphdebug)
			throw "DFSforest: Wrong graph.";
	}
	if (sccEdgeAdded)
	{
		for (int i = 0; i < nVertices; ++ i)
			if (sccEdgeAdded [i] != sccEdgeAddeddebug [i])
				throw "DFSforest: Wrong sccEdgeAdded.";
	}
	if (treeNumber != treeNumberdebug)
		throw "DFSforest: Wrong treeNumber.";
	sbug << "DEBUG: DFSforest OK. ";
	if (!sccGraph)
		sbug << "(Graphs not compared.) ";
	delete [] compVerticesdebug;
	delete [] compEndsdebug;
	if (sccGraphdebug)
		delete sccGraphdebug;
	delete [] ntabdebug;
	delete [] tabdebug;
	if (sccEdgeAddeddebug)
		delete [] sccEdgeAddeddebug;
	#endif // DIGRAPH_DEBUG

	if (sccEdgeAdded)
		delete [] sccEdgeAdded;
	delete [] ntab;
	delete [] tab;
	return countTrees;
} /* diGraph::DFSforest */

template <class wType>
inline int diGraph<wType>::shortestPath (int source, int destination) const
{
	// make sure that the given vertex is present in the graph
	if ((source < 0) || (source >= nVertices) ||
		(destination < 0) || (destination >= nVertices))
	{
		throw "diGraph - shortest path: Wrong vertex number.";
	}

	if (source >= 0)
		throw "Shortest path algorithm not implemented, yet.";

	// initialize minimal nontrivial path lengths to -1 (unvisited)
	std::auto_ptr<int> lengths_ptr (new int [nVertices]);
	int *lengths = lengths_ptr. get ();
	for (int i = 0; i < nVertices; ++ i)
		lengths [i] = -1;

	// prepare stacks for the recursion
	std::stack<int> s_vertex;
	std::stack<int> s_edge;
	std::stack<int> s_maxedge;

	// set the initial vertex
	int vertex = source;
	lengths [vertex] = 0;

	// determine the edges to be visited
	int edge = vertex ? edgeEnds [vertex - 1] : 0;
	int maxedge = edgeEnds [vertex];

	while (1)
	{
		// return to the previous recursion level
		// if all the edges have been followed
		if (edge >= maxedge)
		{
			// return if this is the initial recursion level
			if (s_vertex. empty ())
				return lengths [destination];

			// restore the variables from the previous level
			vertex = s_vertex. top ();
			s_vertex. pop ();
			edge = s_edge. top ();
			s_edge. pop ();
			maxedge = s_maxedge. top ();
			s_maxedge. pop ();
			continue;
		}

		// look at the vertex pointed to by the next edge
		int next = edges [edge ++];

		// update the distance if visited
		if (lengths [next] != -1)
		{
			if ((lengths [next] > lengths [vertex] + 1) ||
				!lengths [next])
			{
				lengths [next] = lengths [vertex] + 1;
			}
		}

		// go to the deeper recursion level if unvisited
		else //if (tab [next] == -1)
		{
			// store the previous variables at the stacks
			s_vertex. push (vertex);
			s_edge. push (edge);
			s_maxedge. push (maxedge);

			// set the path length to the new vertex
			lengths [next] = lengths [vertex] + 1;

			// set the new vertex
			vertex = next;
			
			// determine the edges to be visited
			edge = vertex ? edgeEnds [vertex - 1] : 0;
			maxedge = edgeEnds [vertex];
		}
	}

	return 0;
} /* diGraph::shortestPath */

template <class wType>
inline int diGraph<wType>::shortestLoop (int origin) const
{
	return shortestPath (origin, origin);
} /* diGraph::shortestLoop */

template <class wType>
template <class lenTable, class weightsType, class roundType>
inline void diGraph<wType>::Dijkstra (const roundType &rounding,
	int source, lenTable &len, weightsType &edgeWeights) const
{
	// use the Fibonacci heap as a priority queue
	FibonacciHeap<posWeight> Q (nVertices);

	// add the vertices to the heap with the initial shortest path
	// lengths: 0 to the source, plus infinity to all the others
	for (int v = 0; v < nVertices; ++ v)
	{
		posWeight w (0);
		if (v != source)
			w. setInfinity ();
		int number = Q. Insert (w);
		if (number != v)
		{
			throw "Wrong implementation of Fibonacci heap "
				"for this version of Dijkstra.";
		}
	}

	// pick up vertices from the priority queue, record the length
	// of the shortest path to them, and modify the remaining paths
	for (int i = 0; i < nVertices; ++ i)
	{
		// extract the minimal vertex from the queue
		int minVertex = Q. Minimum ();
		posWeight minWeight = Q. ExtractMinimum ();

		if (minWeight. isInfinity ())
		{
			len [minVertex] = -1;
			continue;
		}
		wType minValue = minWeight. getValue ();
		len [minVertex] = minValue;

		// go through all the edges emanating from this vertex
		// and update the path lengths for the target vertices
		int edge = minVertex ? edgeEnds [minVertex - 1] : 0;
		int maxEdge = edgeEnds [minVertex];
		for (; edge < maxEdge; ++ edge)
		{
			// determine the vertex at the other end of the edge
			int nextVertex = edges [edge];

			// if the path that runs through the extracted
			// vertex is shorter, then make a correction
			const posWeight &nextWeight = Q. Value (nextVertex);
			wType newWeight = rounding. add_down (minValue,
				edgeWeights [edge]);
			if (newWeight < 0)
				newWeight = 0;
			if (nextWeight. isInfinity () ||
				(newWeight < nextWeight. getValue ()))
			{
				Q. DecreaseKey (nextVertex,
					posWeight (newWeight));
			}
		}
	}
	return;
} /* diGraph::Dijkstra */

template <class wType>
template <class lenTable, class roundType>
inline void diGraph<wType>::Dijkstra (const roundType &rounding,
	int source, lenTable &len) const
{
	this -> Dijkstra (rounding, source, len, this -> weights);
	return;
} /* diGraph::Dijkstra */

template <class wType>
template <class lenTable>
inline void diGraph<wType>::Dijkstra (int source, lenTable &len) const
{
	const dummyRounding<wType> rounding = dummyRounding<wType> ();
	this -> Dijkstra (rounding, source, len);
	return;
} /* diGraph::Dijkstra */

template <class wType> template <class lenTable, class predTable,
	class roundType>
inline bool diGraph<wType>::BellmanFord (const roundType &rounding,
	int source, lenTable &len, wType *infinity, predTable pred) const
{
	// make sure the source vertex number is correct
	if ((source < 0) || (source >= nVertices))
		throw "Bellman-Ford: Wrong source vertex number.";

	// prepare marks to indicate finite values (not "infinity")
	BitField finite;
	finite. allocate (nVertices);
	finite. clearall (nVertices);
	finite. set (source);
	len [source] = 0;

	// count the negative vertices
	int countNegative = 0;

	// set the initial predecessors
	for (int i = 0; i < nVertices; ++ i)
		pred [i] = -1;

	// update the lenghts of the paths repeatedly (max nVertices times)
	bool noNegativeLoop = false;
	int counter = 0;
	for (; counter <= nVertices; ++ counter)
	{
		bool modified = false;
		int curEdge = 0;
		for (int vertex = 0; vertex < nVertices; ++ vertex)
		{
			int maxEdge = edgeEnds [vertex];
			if (!finite. test (vertex))
			{
				curEdge = maxEdge;
				continue;
			}
			for (; curEdge < maxEdge; ++ curEdge)
			{
				int next = edges [curEdge];
				wType newlen = rounding. add_down
					(len [vertex], weights [curEdge]);
				if (!finite. test (next))
				{
					finite. set (next);
					modified = true;
					len [next] = newlen;
					pred [next] = vertex;
					if (newlen < 0)
						++ countNegative;
				}
				else if (newlen < len [next])
				{
					modified = true;
					if (!(len [next] < 0) &&
						(newlen < 0))
					{
						++ countNegative;
					}
					len [next] = newlen;
					pred [next] = vertex;
				}
			}
		}
		if (countNegative == nVertices)
		{
			noNegativeLoop = false;
			++ counter;
			break;
		}
		if (!modified)
		{
			noNegativeLoop = true;
			++ counter;
			break;
		}
	}

	// show a message on how many loops have been done
	if (false && chomp::homology::sbug. show)
	{
		chomp::homology::sbug << "Bellman-Ford: " <<
			counter << ((counter > 1) ? " loops (" : " loop (") <<
			nVertices << " vertices, " << countNegative <<
			" negative). " <<
			(noNegativeLoop ? "No negative loops.\n" :
			"A negative loop found.\n");
	}

	// compute the value for the infinity and set the undefined distances
	if (infinity && noNegativeLoop)
	{
		wType infty (0);
		bool first = true;
		for (int i = 0; i < nVertices; ++ i)
		{
			if (!finite. test (i))
				continue;
			if (first)
			{
				infty = len [i];
				first = false;
			}
			else if (infty < len [i])
			{
				infty = len [i];
			}
		}
		infty = infty + 1;
		for (int i = 0; i < nVertices; ++ i)
		{
			if (!finite. test (i))
				len [i] = infty;
		}
		*infinity = infty;
	}

	finite. free ();
	return noNegativeLoop;
} /* diGraph::BellmanFord */

template <class wType> template <class lenTable, class predTable>
inline bool diGraph<wType>::BellmanFord (int source, lenTable &len,
	wType *infinity, predTable pred) const
{
	const dummyRounding<wType> rounding = dummyRounding<wType> ();
	return this -> BellmanFord (rounding, source, len, infinity, pred);
} /* diGraph::BellmanFord */

template <class wType> template <class roundType>
inline bool diGraph<wType>::BellmanFord (const roundType &rounding,
	int source) const
{
	std::auto_ptr<wType> len_ptr (new wType [nVertices]);
	wType *len = len_ptr. get ();
	wType *infinity = 0;
	dummyArray tab;
	return BellmanFord (rounding, source, len, infinity, tab);
} /* diGraph::BellmanFord */

template <class wType>
inline bool diGraph<wType>::BellmanFord (int source) const
{
	const dummyRounding<wType> rounding = dummyRounding<wType> ();
	return BellmanFord (rounding, source);
} /* diGraph::BellmanFord */

template <class wType>
inline wType diGraph<wType>::Edmonds () const
{
	// create a list of edges with weights and sort this list
	std::vector<edgeTriple> edgeTriples (countEdges ());
	int prevEdge = 0;
	int curEdge = 0;
	for (int vert = 0; vert < nVertices; ++ vert)
	{
		while (curEdge < edgeEnds [vert])
		{
			edgeTriple &e = edgeTriples [curEdge];
			e. vert1 = vert;
			e. vert2 = edges [curEdge];
			e. weight = weights [curEdge];
			++ curEdge;
		}
		prevEdge = curEdge;
	}
	std::sort (edgeTriples. begin (), edgeTriples. end ());

	// create a forest which initially contains single vertices
	std::auto_ptr<int> root_ptr (new int [nVertices]);
	int *root = root_ptr. get ();
	for (int vert = 0; vert < nVertices; ++ vert)
	{
		root [vert] = -1;
	}

	// go through the edges and join the trees, but avoid loops
	wType totalWeight = 0;
	int nEdges = countEdges ();
	for (int curEdge = 0; curEdge < nEdges; ++ curEdge)
	{
		// determine the roots of both vertices of the edge
		// and compress the paths
		edgeTriple &e = edgeTriples [curEdge];
		int root1 = e. vert1;
		while (root [root1] >= 0)
		{
			root1 = root [root1];
		}
		int vert1 = e. vert1;
		while (root [vert1] >= 0)
		{
			int next = root [vert1];
			root [vert1] = root1;
			vert1 = next;
		}
		int root2 = e. vert2;
		while (root [root2] >= 0)
		{
			root2 = root [root2];
		}
		int vert2 = e. vert2;
		while (root [vert2] >= 0)
		{
			int next = root [vert2];
			root [vert2] = root2;
			vert2 = next;
		}

		// skip the edge if it closes a loop
		if (root1 == root2)
			continue;

		// add the weight
		totalWeight += e. weight;

		// join the trees
		root [root1] = root2;
	}
	return totalWeight;
} /* diGraph::Edmonds */

template <class wType>
inline wType diGraph<wType>::EdmondsOld () const
{
	// create a list of edges with weights and sort this list
	std::vector<edgeTriple> edgeTriples (countEdges ());
	int prevEdge = 0;
	int curEdge = 0;
	for (int vert = 0; vert < nVertices; ++ vert)
	{
		while (curEdge < edgeEnds [vert])
		{
			edgeTriple &e = edgeTriples [curEdge];
			e. vert1 = vert;
			e. vert2 = edges [curEdge];
			e. weight = weights [curEdge];
			++ curEdge;
		}
		prevEdge = curEdge;
	}
	std::sort (edgeTriples. begin (), edgeTriples. end ());

	// create a forest which initially contains single vertices
	std::auto_ptr<int> forest_ptr (new int [nVertices]);
	int *forest = forest_ptr. get ();
	std::auto_ptr<int> next_ptr (new int [nVertices]);
	int *next = next_ptr. get ();
	std::auto_ptr<int> prev_ptr (new int [nVertices]);
	int *prev = prev_ptr. get ();
	for (int vert = 0; vert < nVertices; ++ vert)
	{
		forest [vert] = vert;
		next [vert] = -1;
		prev [vert] = -1;
	}

	// go through the edges and join the trees, but avoid loops
	wType totalWeight = 0;
	int nEdges = countEdges ();
	for (int curEdge = 0; curEdge < nEdges; ++ curEdge)
	{
		// check the edge and skip it if it closes a loop
		edgeTriple &e = edgeTriples [curEdge];
		if (forest [e. vert1] == forest [e. vert2])
			continue;

		// add the weight
		totalWeight += e. weight;

		// go to the end of the first tree
		int vert1 = e. vert1;
		while (next [vert1] >= 0)
		{
			vert1 = next [vert1];
		}
		
		// go to the beginning of the second tree
		int vert2 = e. vert2;
		while (prev [vert2] >= 0)
		{
			vert2 = prev [vert2];
		}

		// join the trees and modify the numbers
		next [vert1] = vert2;
		prev [vert2] = vert1;
		int treeNumber = forest [vert1];
		while (vert2 >= 0)
		{
			forest [vert2] = treeNumber;
			vert2 = next [vert2];
		}
	}
	return totalWeight;
} /* diGraph::EdmondsOld */

template <class wType>
template <class arrayType, class roundType>
inline wType diGraph<wType>::FloydWarshall (const roundType &rounding,
	arrayType &arr, bool setInfinity, bool ignoreNegLoop) const
{
	// do nothing if the graph is empty
	if (!nVertices)
		return 0;

	// prepare marks to indicate finite values (not "infinity")
	BitField *finite = new BitField [nVertices];
	for (int i = 0; i < nVertices; ++ i)
	{
		finite [i]. allocate (nVertices);
		finite [i]. clearall (nVertices);
	}

	// create the initial values of the array based on the edge weights
	int curEdge = 0;
	for (int source = 0; source < nVertices; ++ source)
	{
		bool diagset = false;
		while (curEdge < edgeEnds [source])
		{
			int target = edges [curEdge];
			const wType &weight = weights [curEdge];
			if (source == target)
			{
				if (weight < 0)
				{
					arr [source] [target] = weight;
					diagset = true;
				}
			}
			else
			{
				arr [source] [target] = weight;
				finite [source]. set (target);
			}
			++ curEdge;
		}
		if (!diagset)
			arr [source] [source] = 0;
		finite [source]. set (source);
	}

	// find the shortest paths between the vertices (dynamic programming)
	for (int k = 0; k < nVertices; ++ k)
	{
		for (int i = 0; i < nVertices; ++ i)
		{
			if (!finite [i]. test (k))
				continue;
			for (int j = 0; j < nVertices; ++ j)
			{
				if (!finite [k]. test (j))
					continue;
				const wType sum = rounding. add_down
					(arr [i] [k], arr [k] [j]);
				if (finite [i]. test (j))
				{
					if (sum < arr [i] [j])
						arr [i] [j] = sum;
				}
				else
				{
					arr [i] [j] = sum;
					finite [i]. set (j);
				}
			}
		}
	}

	// verify if a negative loop exists by checking for a negative
	// result in the diagonal
	if (!ignoreNegLoop)
	{
		for (int i = 0; i < nVertices; ++ i)
		{
			if (arr [i] [i] < 0)
				throw "Negative loop in Floyd-Warshall.";
		}
	}

	// prepare a variable to store the returned result
	wType result = 0;

	// compute the value for the infinity and fill in the array
	// if requested to do so
	if (setInfinity)
	{
		wType &infinity = result;
		for (int i = 0; i < nVertices; ++ i)
		{
			for (int j = 0; j < nVertices; ++ j)
			{
				if (finite [i]. test (j) &&
					(infinity <= arr [i] [j]))
				{
					infinity = rounding. add_up
						(arr [i] [j], 1);
				}
			}
		}
		for (int i = 0; i < nVertices; ++ i)
		{
			for (int j = 0; j < nVertices; ++ j)
			{
				if (!finite [i]. test (j))
					arr [i] [j] = infinity;
			}
		}
	}

	// otherwise, only compute the minimum path weight
	else
	{
		wType &minWeight = result;
		bool firstTime = true;
		for (int i = 0; i < nVertices; ++ i)
		{
			for (int j = 0; j < nVertices; ++ j)
			{
				if (finite [i]. test (j))
				{
					if (firstTime)
					{
						minWeight = arr [i] [j];
						firstTime = false;
					}
					else if (arr [i] [j] < minWeight)
					{
						minWeight = arr [i] [j];
					}
				}
			}
		}
	}

	// release the 'finite' bitfields
	for (int i = 0; i < nVertices; ++ i)
		finite [i]. free ();
	delete [] finite;

	return result;
} /* diGraph::FloydWarshall */

template <class wType>
template <class arrayType>
inline wType diGraph<wType>::FloydWarshall (arrayType &arr,
	bool setInfinity, bool ignoreNegLoop) const
{
	const dummyRounding<wType> rounding = dummyRounding<wType> ();
	return FloydWarshall (rounding, arr, setInfinity, ignoreNegLoop);
} /* diGraph::FloydWarshall */

template <class wType>
template <class arrayType, class roundType>
inline wType diGraph<wType>::Johnson (const roundType &rounding,
	arrayType &arr, bool setInfinity, bool ignoreNegLoop) const
{
	// DEBUG VERIFICATION
	if (false && sbug. show)
	{
		timeused stopWatch;
		wType res = FloydWarshall (rounding,
			arr, setInfinity, ignoreNegLoop);
		chomp::homology::sbug << "\n[Floyd-Warshall: " << res <<
			", " << (double) stopWatch << " sec]\n";
	}
	// debug time measurement (see below)
//	timeused stopWatch;

	// do nothing if the graph is empty
	if (!nVertices)
		return 0;

	// STEP 1: Compute the shortest paths to any vertex from an
	// artificial extra vertex connected to every other vertex in the
	// graph by an edge of weight zero. Use Bellman-Ford for this.
	wType *len = new wType [nVertices];
	for (int i = 0; i < nVertices; ++ i)
		len [i] = 0;

	// update the lenghts of the paths repeatedly (max nVertices times)
	bool noNegativeLoop = false;
	int counter = 0;
	for (; counter <= nVertices; ++ counter)
	{
		bool modified = false;
		int curEdge = 0;
		for (int vertex = 0; vertex < nVertices; ++ vertex)
		{
			int maxEdge = edgeEnds [vertex];
			for (; curEdge < maxEdge; ++ curEdge)
			{
				int next = edges [curEdge];
				wType newlen = rounding. add_down
					(len [vertex], weights [curEdge]);
				if (newlen < len [next])
				{
					// this "if" statement is necessary
					// because of a bug in GCC 3.4.2...
					if (counter > nVertices)
					{
						std::cout << vertex;
					}
					modified = true;
					len [next] = newlen;
				}
			}
		}
		if (!modified)
		{
			noNegativeLoop = true;
			++ counter;
			break;
		}
	}
	if (!ignoreNegLoop && !noNegativeLoop)
		throw "Negative loop found in Johnson's algorithm.";

	// STEP 2: Re-weigh the edges using the computed path lengths.
	wType *newWeights = new wType [edgeEnds [nVertices - 1]];
	int edge = 0;
	for (int vertex = 0; vertex < nVertices; ++ vertex)
	{
		int maxEdge = edgeEnds [vertex];
		for (; edge < maxEdge; ++ edge)
		{
			wType w = weights [edge];
			w = rounding. add_down (w, len [vertex]);
			w = rounding. sub_down (w, len [edges [edge]]);
			newWeights [edge] = (w < 0) ? 0 : w;
		}
	}

	// STEP 3: Run the Dijkstra algorithm for every vertex to compute
	// the shortest paths to all the other vertices.
	// Negative entries indicate no path existence.
	for (int u = 0; u < nVertices; ++ u)
	{
		this -> Dijkstra (rounding, u, arr [u], newWeights);
	}
	delete [] newWeights;

	// STEP 4: Make a correction to the computed path lengths.
	// Compute the value for infinity if requested to.
	// Otherwise compute the minimum of path lengths.
	wType result = 0;
	if (setInfinity)
	{
		wType &infinity = result;
		for (int u = 0; u < nVertices; ++ u)
		{
			for (int v = 0; v < nVertices; ++ v)
			{
				wType w = arr [u] [v];
				if (w < 0)
					continue;
				w = rounding. add_down (w, len [v]);
				w = rounding. sub_down (w, len [u]);
				if (w < infinity)
					continue;
				infinity = rounding. add_up (w, 1);
			}
		}
		for (int u = 0; u < nVertices; ++ u)
		{
			for (int v = 0; v < nVertices; ++ v)
			{
				wType w = arr [u] [v];
				if (w < 0)
				{
					arr [u] [v] = infinity;
					continue;
				}
				w = rounding. add_down (w, len [v]);
				arr [u] [v] =
					rounding. sub_down (w, len [u]);
			}
		}
	}
	else
	{
		wType &minWeight = result;
		bool firstTime = true;
		for (int u = 0; u < nVertices; ++ u)
		{
			for (int v = 0; v < nVertices; ++ v)
			{
				wType w = arr [u] [v];
				if (w < 0)
					continue;
				w = rounding. add_down (w, len [v]);
				w = rounding. sub_down (w, len [u]);
				if (firstTime)
				{
					minWeight = w;
					firstTime = false;
				}
				else if (w < minWeight)
					minWeight = w;
			}
		}
	}
	delete [] len;

	// DEBUG VERIFICATION
	if (false && sbug. show)
	{
//		chomp::homology::sbug << "[Johnson: " << result <<
//			", " << (double) stopWatch << " sec]\n";
	}

	return result;
} /* diGraph::Johnson */

template <class wType>
template <class arrayType>
inline wType diGraph<wType>::Johnson (arrayType &arr,
	bool setInfinity, bool ignoreNegLoop) const
{
	const dummyRounding<wType> rounding = dummyRounding<wType> ();
	return Johnson (rounding, arr, setInfinity, ignoreNegLoop);
} /* diGraph::Johnson */

template <class wType>
template <class roundType>
wType diGraph<wType>::minPathWeight (const roundType &rounding,
	bool ignoreNegLoop, int sparseGraph) const
{
	// if the graph is empty, return 0 as the minimum path weight
	if (nVertices <= 0)
		return 0;

	// allocate memory for an array of arrays to store min path weights
	wType **arr = new wType * [nVertices];
	for (int i = 0; i < nVertices; ++ i)
		arr [i] = new wType [nVertices];

	// determine whether to run the Floyd-Warshall algorithm
	// or Johnson's algorithm
	bool sparse = false;
	if (sparseGraph == 1)
		sparse = true;
	else if (sparseGraph != 0)
	{
		double nEdgesD = this -> countEdges ();
		double nVerticesD = this -> countVertices ();
		if ((nVerticesD > 3000) && (nEdgesD < nVerticesD * 1000) &&
			(nEdgesD < nVerticesD * nVerticesD / 50))
		{
			sparse = true;
		}
	}

	// run the Johnson's or Floyd-Warshall algorithm
	wType minWeight = sparse ?
		this -> Johnson (rounding, arr, false, ignoreNegLoop) :
		this -> FloydWarshall (rounding, arr, false, ignoreNegLoop);

/*	// compute the minimum of all the paths
	wType minWeight = arr [0] [0];
	for (int i = 0; i < nVertices; ++ i)
	{
		for (int j = 0; j < nVertices; ++ j)
		{
			const wType &weight = arr [i] [j];
			if (weight < minWeight)
				minWeight = weight;
		}
	}
*/
	// release the memory
	for (int i = 0; i < nVertices; ++ i)
		delete [] (arr [i]);
	delete [] arr;

	return minWeight;
} /* diGraph::minPathWeight */

template <class wType>
wType diGraph<wType>::minPathWeight (bool ignoreNegLoop, int sparseGraph)
	const
{
	const dummyRounding<wType> rounding = dummyRounding<wType> ();
	return this -> minPathWeight (rounding, ignoreNegLoop, sparseGraph);
} /* diGraph::minPathWeight */

template <class wType> template <class outType>
inline outType &diGraph<wType>::show (outType &out, bool showWeights) const
{
	out << "; Directed graph: " << nVertices << " vertices.\n";
	int curEdge = 0;
	for (int i = 0; i < nVertices; ++ i)
	{
		for (; curEdge < edgeEnds [i]; ++ curEdge)
		{
			out << i << " -> " << edges [curEdge];
			if (showWeights)
				out << " [" << weights [curEdge] << "]\n";
			else
				out << "\n";
		}
	}
	out << "; This is the end of the graph.\n";
	return out;
} /* diGraph::show */

	
// --------------------------------------------------

/// Writes a graph in the text mode to an output stream.
template <class wType>
inline std::ostream &operator << (std::ostream &out, const diGraph<wType> &g)
{
	return g. show (out, false);
} /* operator << */

// --------------------------------------------------

/// Computes strongly connected components of the graph 'g'. Creates
/// the graph 'scc' in which each vertex corresponds to one component.
/// The graph 'scc' given as an argument must be initially empty.
/// The table 'compVertices' is filled with the numbers of vertices in 'g'
/// which form the components, and the indices that end the listing
/// for each component are stored in the table 'compEnds'.
/// Returns the number of strongly connected components found.
template <class wType, class Table1, class Table2>
inline int SCC (const diGraph<wType> &g, Table1 &compVertices,
	Table2 &compEnds, diGraph<wType> *scc = 0,
	diGraph<wType> *transposed = 0, bool copyweights = false)
{
	// prepare two tables
	int nVert = g. countVertices ();
	int *ordered = new int [nVert];
	int *tab = new int [nVert];

	// compute the list of vertices in the descending finishing time
	g. DFSfinishTime (tab);
	for (int i = 0; i < nVert; ++ i)
		ordered [nVert - tab [i]] = i;
	delete [] tab;

	// create the transposed graph
	diGraph<wType> gT;
	if (!transposed)
		transposed = &gT;
	g. transpose (*transposed, copyweights);

	// extract the DFS forest of gT in the given order of vertices
	int n = transposed -> DFSforest (ordered, compVertices, compEnds,
		true, scc);

	// cleanup memory and return the number of components
	delete [] ordered;
	return n;
} /* SCC */

// --------------------------------------------------

/// Computes strongly connected components of the graph 'g'
/// using Tarjan's algorithm (as described in the Wikipedia).
/// Tha advantage of this approach over the one described in Cormen's
/// textbook is that the transposed graph need not be computed.
/// However, this algorithm might be slightly slower than the other one.
/// The table 'compVertices' is filled with the numbers of vertices in 'g'
/// which form the components, and the indices that end the listing
/// for each component are stored in the table 'compEnds'.
/// Returns the number of strongly connected components found.
template <class wType, class Table1, class Table2>
inline int SCC_Tarjan (const diGraph<wType> &g, Table1 &compVertices,
	Table2 &compEnds)
{
	// return the obvious result if the graph is empty
	int nVertices = g. countVertices ();
	if (!nVertices)
		return 0;

	// prepare an array of discovery times for all the vertices
	// (zero == not yet discovered)
	std::vector<int> dfsIndex (nVertices, 0);

	// prepare an array of the minimal index of a node reachable
	// from each of the vertices
	std::vector<int> lowLink (nVertices, 0);

	// prepare an empty stack of nodes
	std::stack<int> s_nodes;

	// prepare an array of bits indicating whether the vertices are
	// in the stack of nodes or not
	BitField inTheStack;
	inTheStack. allocate (nVertices);
	inTheStack. clearall (nVertices);

	// prepare the number of strongly connected components
	int nComponents = 0;

	// prepare the current position in the array 'compVertices'
	int posVertices = 0;

	// remember the next vertex number in the graph to scan
	// whether this vertex has already been visited or not
	int vertexToScan = 0;

	// prepare a variable for storing the discovery time in the DFS
	int discoveryTime = 0;

	// prepare stacks for the DFS recursion
	std::stack<int> s_vertex;
	std::stack<int> s_edge;
	std::stack<int> s_maxedge;

	// initialize the number of the currently processed vertex
	int vertex = -1;

	// initialize the range of edges to be visited
	int edge = 0;
	int maxedge = 0;

	while (1)
	{
		// return to the previous recursion level
		// if all the edges have been checked
		if (edge >= maxedge)
		{
			// extract a strongly connected component
			if ((vertex >= 0) && (lowLink [vertex] ==
				dfsIndex [vertex]))
			{
				int v = 0;
				do
				{
					v = s_nodes. top ();
					s_nodes. pop ();
					inTheStack. clear (v);
					compVertices [posVertices ++] = v;
				} while (v != vertex);
				compEnds [nComponents ++] = posVertices;
			}

			// if this is the top level of the recursion
			// then find another unvisited vertex or return
			if (s_vertex. empty ())
			{
				// find an unvisited vertex in the graph
				while ((vertexToScan < nVertices) &&
					(dfsIndex [vertexToScan] != 0))
				{
					++ vertexToScan;
				}

				// return the result if all visited
				if (vertexToScan == nVertices)
				{
					inTheStack. free ();
					return nComponents;
				}

				// set the new vertex
				vertex = vertexToScan ++;

				// mark the current vertex as visited
				// and initialize its low link
				dfsIndex [vertex] = ++ discoveryTime;
				lowLink [vertex] = discoveryTime;

				// push this vertex on the stack
				s_nodes. push (vertex);
				inTheStack. set (vertex);

				// determine the edges to be visited
				edge = 0;
				maxedge = g. countEdges (vertex);
			}

			// otherwise trace back to the previous level
			else
			{
				// remember the current low link index
				int lowLink2 = lowLink [vertex];

				// restore the variables
				vertex = s_vertex. top ();
				s_vertex. pop ();
				edge = s_edge. top ();
				s_edge. pop ();
				maxedge = s_maxedge. top ();
				s_maxedge. pop ();

				// update the current low link index
				if (lowLink [vertex] > lowLink2)
					lowLink [vertex] = lowLink2;
			}
		}

		// analyse the next edge coming out from the current vertex
		else
		{
			// determine the next vertex
			int next = g. getEdge (vertex, edge ++);

			// go to a deeper recursion level if unvisited
			if (dfsIndex [next] == 0)
			{
				// store the variables at the stacks
				s_vertex. push (vertex);
				s_edge. push (edge);
				s_maxedge. push (maxedge);

				// set the new vertex
				vertex = next;
			
				// mark the new vertex as visited
				dfsIndex [vertex] = ++ discoveryTime;
				lowLink [vertex] = discoveryTime;
			
				// push this vertex on the stack
				s_nodes. push (vertex);
				inTheStack. set (vertex);

				// determine the edges to be visited
				edge = 0;
				maxedge = g. countEdges (vertex);
			}

			// update the low link index if the vertex has been
			// visited and is currently in the stack of nodes
			else if (inTheStack. test (next))
			{
				if (lowLink [vertex] > dfsIndex [next])
					lowLink [vertex] = dfsIndex [next];
			}
		}
	}

	// finalize and return the number of strongly connected components
	inTheStack. free ();
	return nComponents;
} /* SCC_Tarjan */

// --------------------------------------------------

template <class wType>
wType diGraph<wType>::minMeanCycleWeight (diGraph<wType> *transposed) const
{
	// find the strongly connected components of the graph
	multitable<int> compVertices, compEnds;
	bool copyweights = !!transposed;
	int countSCC = SCC (*this, compVertices, compEnds,
		static_cast<diGraph<wType> *> (0), transposed, copyweights);
	if (countSCC <= 0)
		return 0;

	// compute the maximum size of each strongly connected component
	int maxCompSize = compEnds [0];
	for (int comp = 1; comp < countSCC; ++ comp)
	{
		int compSize = compEnds [comp] - compEnds [comp - 1];
		if (maxCompSize < compSize)
			maxCompSize = compSize;
	}

	// allocate arrays for weights and bit fields
	wType **F = new wType * [maxCompSize + 1];
	BitField *finite = new BitField [maxCompSize + 1];
	for (int i = 0; i <= maxCompSize; ++ i)
	{
		F [i] = new wType [maxCompSize];
		finite [i]. allocate (maxCompSize);
	}

	// compute the numbers of vertices in each component
	int *numbers = new int [nVertices];
	int *components = new int [nVertices];
	for (int i = 0; i < nVertices; ++ i)
		components [i] = -1;
	int offset = 0;
	for (int comp = 0; comp < countSCC; ++ comp)
	{
		int maxOffset = compEnds [comp];
		int pos = offset;
		for (; pos < maxOffset; ++ pos)
		{
			numbers [compVertices [pos]] = pos - offset;
			components [compVertices [pos]] = comp;
		}
		offset = pos;
	}

	// compute the minimum mean cycle weight for each component
	wType minWeight (0);
	for (int comp = 0; comp < countSCC; ++ comp)
	{
		int offset = comp ? compEnds [comp - 1] : 0;
		int compSize = compEnds [comp] - offset;
		for (int i = 0; i <= compSize; ++ i)
			finite [i]. clearall (compSize);
		F [0] [0] = 0;
		finite [0]. set (0);
		// compute path progressions of given length
		for (int len = 1; len <= compSize; ++ len)
		{
			// process source vertices
			for (int i = 0; i < compSize; ++ i)
			{
				if (!finite [len - 1]. test (i))
					continue;
				wType prevWeight = F [len - 1] [i];
				int source = compVertices [offset + i];

				// process target vertices (and edges)
				int edgeOffset = source ?
					edgeEnds [source - 1] : 0;
				int edgeEnd = edgeEnds [source];
				for (; edgeOffset < edgeEnd; ++ edgeOffset)
				{
					int target = edges [edgeOffset];
					if (components [target] != comp)
						continue;
					int j = numbers [target];
					wType newWeight = prevWeight +
						weights [edgeOffset];
					if (!finite [len]. test (j))
					{
						finite [len]. set (j);
						F [len] [j] = newWeight;
					}
					else if (newWeight < F [len] [j])
						F [len] [j] = newWeight;
				}
			}
		}

		// compute the minimum mean cycle weight for this component
		wType minCompWeight (0);
		bool firstMin = true;
		for (int vert = 0; vert < compSize; ++ vert)
		{
			if (!finite [compSize]. test (vert))
				continue;
			bool firstAverage = true;
			wType maxAverage = 0;
			for (int first = 0; first < compSize; ++ first)
			{
				if (!finite [first]. test (vert))
					continue;
				wType average = (F [compSize] [vert] -
					F [first] [vert]) /
					(compSize - first);
				if (firstAverage)
				{
					maxAverage = average;
					firstAverage = false;
				}
				else if (maxAverage < average)
					maxAverage = average;
			}
			if (firstMin || (maxAverage < minCompWeight))
			{
				if (firstAverage)
					throw "Bug in Karp's algorithm";
				minCompWeight = maxAverage;
				firstMin = false;
			}
		}

		// make a correction to the total minimum if necessary
		if (!comp || (minCompWeight < minWeight))
			minWeight = minCompWeight;
	}

	// release allocated memory
	delete [] components;
	delete [] numbers;
	for (int i = 0; i < maxCompSize; ++ i)
	{
		finite [i]. free ();
		delete [] (F [i]);
	}
	delete [] finite;
	delete [] F;

	// return the computed minimum
	return minWeight;
} /* diGraph::minMeanCycleWeight */

template <class wType>
template <class roundType>
wType diGraph<wType>::minMeanCycleWeight (const roundType &rounding,
	diGraph<wType> *transposed) const
{
	// find the strongly connected components of the graph
	multitable<int> compVertices, compEnds;
	bool copyweights = !!transposed;
	int countSCC = SCC (*this, compVertices, compEnds,
		static_cast<diGraph<wType> *> (0), transposed, copyweights);
	if (countSCC <= 0)
		return 0;

	// compute the maximum size of each strongly connected component
	int maxCompSize = compEnds [0];
	for (int comp = 1; comp < countSCC; ++ comp)
	{
		int compSize = compEnds [comp] - compEnds [comp - 1];
		if (maxCompSize < compSize)
			maxCompSize = compSize;
	}

	// allocate arrays for weights and bit fields
	wType **FUpper = new wType * [maxCompSize + 1];
	wType **FLower = new wType * [maxCompSize + 1];
	BitField *finite = new BitField [maxCompSize + 1];
	for (int i = 0; i <= maxCompSize; ++ i)
	{
		FUpper [i] = new wType [maxCompSize];
		FLower [i] = new wType [maxCompSize];
		finite [i]. allocate (maxCompSize);
	}

	// compute the numbers of vertices in each component
	int *numbers = new int [nVertices];
	int *components = new int [nVertices];
	for (int i = 0; i < nVertices; ++ i)
		components [i] = -1;
	int offset = 0;
	for (int comp = 0; comp < countSCC; ++ comp)
	{
		int maxOffset = compEnds [comp];
		int pos = offset;
		for (; pos < maxOffset; ++ pos)
		{
			numbers [compVertices [pos]] = pos - offset;
			components [compVertices [pos]] = comp;
		}
		offset = pos;
	}

	// compute the minimum mean cycle weight for each component
	wType minWeight (0);
	for (int comp = 0; comp < countSCC; ++ comp)
	{
		int offset = comp ? compEnds [comp - 1] : 0;
		int compSize = compEnds [comp] - offset;
		for (int i = 0; i <= compSize; ++ i)
			finite [i]. clearall (compSize);
		FUpper [0] [0] = 0;
		FLower [0] [0] = 0;
		finite [0]. set (0);
		// compute path progressions of given length
		for (int len = 1; len <= compSize; ++ len)
		{
			// process source vertices
			for (int i = 0; i < compSize; ++ i)
			{
				if (!finite [len - 1]. test (i))
					continue;
				wType prevUpper = FUpper [len - 1] [i];
				wType prevLower = FLower [len - 1] [i];
				int source = compVertices [offset + i];

				// process target vertices (and edges)
				int edgeOffset = source ?
					edgeEnds [source - 1] : 0;
				int edgeEnd = edgeEnds [source];
				for (; edgeOffset < edgeEnd; ++ edgeOffset)
				{
					int target = edges [edgeOffset];
					if (components [target] != comp)
						continue;
					int j = numbers [target];
					wType newUpper = rounding. add_up
						(prevUpper,
						weights [edgeOffset]);
					wType newLower = rounding. add_down
						(prevLower,
						weights [edgeOffset]);
					if (!finite [len]. test (j))
					{
						finite [len]. set (j);
						FUpper [len] [j] = newUpper;
						FLower [len] [j] = newLower;
					}
					else
					{
						wType &curUpper =
							FUpper [len] [j];
						if (newUpper < curUpper)
							curUpper = newUpper;
						wType &curLower =
							FLower [len] [j];
						if (newLower < curLower)
							curLower = newLower;
					}
				}
			}
		}

		// compute the minimum mean cycle weight for this component
		wType minCompWeight (0);
		bool firstMin = true;
		for (int vert = 0; vert < compSize; ++ vert)
		{
			if (!finite [compSize]. test (vert))
				continue;
			bool firstAverage = true;
			wType maxAverage = 0;
			for (int first = 0; first < compSize; ++ first)
			{
				if (!finite [first]. test (vert))
					continue;
				const wType diff = rounding. sub_down
					(FLower [compSize] [vert],
					FUpper [first] [vert]);
				wType average = rounding. div_down
					(diff, compSize - first);
				if (firstAverage)
				{
					maxAverage = average;
					firstAverage = false;
				}
				else if (maxAverage < average)
					maxAverage = average;
			}
			if (firstMin || (maxAverage < minCompWeight))
			{
				if (firstAverage)
					throw "Bug in Karp's algorithm";
				minCompWeight = maxAverage;
				firstMin = false;
			}
		}

		// make a correction to the total minimum if necessary
		if (!comp || (minCompWeight < minWeight))
			minWeight = minCompWeight;
	}

	// release allocated memory
	delete [] components;
	delete [] numbers;
	for (int i = 0; i < maxCompSize; ++ i)
	{
		finite [i]. free ();
		delete [] (FUpper [i]);
		delete [] (FLower [i]);
	}
	delete [] finite;
	delete [] FUpper;
	delete [] FLower;

	// return the computed minimum
	return minWeight;
} /* diGraph::minMeanCycleWeight_intv */

template <class wType>
template <class arrayType, class roundType>
wType diGraph<wType>::minMeanPathWeight (const roundType &rounding,
	const arrayType &starting, int n) const
{
	// allocate arrays for weights and bit fields
	int nIndices = 2;
	wType **F = new wType * [nIndices];
	BitField *finite = new BitField [nIndices];
	for (int i = 0; i < nIndices; ++ i)
	{
		F [i] = new wType [nVertices];
		finite [i]. allocate (nVertices);
	}

	// set the zero path lengths from the initial vertices
	for (int i = 0; i < n; ++ i)
	{
		int v = starting [i];
		if ((v < 0) || (v >= nVertices))
			throw "Starting vertex out of range.";
		F [0] [v] = 0;
		finite [0]. set (v);
	}

	// compute path progressions of given length and average weights
	wType minWeight (0);
	bool firstWeight = true;
	for (int len = 1; len <= nVertices; ++ len)
	{
		// determine the indices for previous and current paths
		int prevIndex = (len - 1) & 1;
		int curIndex = len & 1;
		finite [curIndex]. clearall (nVertices);

		// process source vertices
		for (int source = 0; source < nVertices; ++ source)
		{
			if (!finite [prevIndex]. test (source))
				continue;
			wType prevWeight = F [prevIndex] [source];

			// process target vertices (and edges)
			int edgeOffset = source ?
				edgeEnds [source - 1] : 0;
			int edgeEnd = edgeEnds [source];
			for (; edgeOffset < edgeEnd; ++ edgeOffset)
			{
				int target = edges [edgeOffset];
				wType newWeight = rounding. add_down
					(prevWeight, weights [edgeOffset]);
				if (!finite [curIndex]. test (target))
				{
					finite [curIndex]. set (target);
					F [curIndex] [target] = newWeight;
				}
				else if (newWeight < F [curIndex] [target])
					F [curIndex] [target] = newWeight;
			}
		}

		// update the minimum mean path weight
		for (int vert = 0; vert < nVertices; ++ vert)
		{
			if (!finite [curIndex]. test (vert))
				continue;
			wType average = rounding. div_down
				(F [curIndex] [vert], len);
			if (firstWeight)
			{
				minWeight = average;
				firstWeight = false;
			}
			else if (average < minWeight)
				minWeight = average;
		}
	}

	// release allocated memory
	for (int i = 0; i < nIndices; ++ i)
	{
		finite [i]. free ();
		delete [] (F [i]);
	}
	delete [] finite;
	delete [] F;

	// return the computed minimum
	return minWeight;
} /* diGraph::minMeanPathWeight */

template <class wType>
template <class arrayType>
wType diGraph<wType>::minMeanPathWeight (const arrayType &starting, int n)
	const
{
	const dummyRounding<wType> rounding = dummyRounding<wType> ();
	return minMeanPathWeight (rounding, starting, n);
} /* diGraph::minMeanPathWeight */


// --------------------------------------------------
// ------------- OTHER GRAPH ALGORITHMS -------------
// --------------------------------------------------

// A helper function for "collapseVertices" (see below).
template <class wType>
inline int addRenumEdges (const diGraph<wType> &g, int vertex,
	const int *newNum, int cur, int *srcVert, diGraph<wType> &result)
{
	int nEdges = g. countEdges (vertex);
	// add all the edges that start at the component
	for (int edge = 0; edge < nEdges; ++ edge)
	{
		// determine the dest. vertex of the edge
		int dest = newNum [g. getEdge (vertex, edge)];
		// if this is an edge to self, then ignore it
		if (dest == cur)
			continue;
		// if the edge has already been added,
		// then skip it
		if (srcVert [dest] == cur)
			continue;
		// remember that the dest. vertex has an edge
		// pointing out from the current vertex
		srcVert [dest] = cur;
		// add the edge to the result graph
		result. addEdge (dest);
	}
	return 0;
} /* addRenumEdges */

/// Collapses sets of vertices to single vertices and creates
/// a corresponding graph in which the first vertices come from
/// the collapsed ones. The result graph must be initially empty.
/// Saves translation table from old to new vertex numbers.
/// The table must have sufficient length.
template <class wType, class Table1, class Table2>
inline int collapseVertices (const diGraph<wType> &g,
	int compNum, Table1 &compVertices, Table2 &compEnds,
	diGraph<wType> &result, int *newNumbers = 0)
{
	// do nothing if the input graph is empty
	int nVert = g. countVertices ();
	if (!nVert)
		return 0;

	// compute the new numbers of vertices: newNum [i] is the
	// number of vertex 'i' from 'g' in the resulting graph
	int *newNum = newNumbers ? newNumbers : new int [nVert];
	for (int i = 0; i < nVert; ++ i)
		newNum [i] = -1;
	int cur = 0; // current vertex number in the new graph
	int pos = 0; // position in compVertices
	while (cur < compNum)
	{
		int posEnd = compEnds [cur];
		while (pos < posEnd)
			newNum [compVertices [pos ++]] = cur;
		++ cur;
	}
	for (int i = 0; i < nVert; ++ i)
	{
		if (newNum [i] < 0)
			newNum [i] = cur ++;
	}

	// prepare a table to mark the most recent source vertex for edges:
	// srcVert [j] contains i if the edge i -> j has just been added
	int newVert = nVert - (compNum ? compEnds [compNum - 1] : 0) +
		compNum;
	// debug:
	if (cur != newVert)
		throw "DEBUG: Wrong numbers.";
	int *srcVert = new int [newVert];
	for (int i = 0; i < newVert; ++ i)
		srcVert [i] = -1;

	// scan the input graph and create the resulting graph: begin with
	// the vertices in the collapsed groups
	cur = 0, pos = 0;
	while (cur < compNum)
	{
		// add a new vertex to the result graph
		result. addVertex ();
		// for all the vertices in this component...
		int posEnd = compEnds [cur];
		while (pos < posEnd)
		{
			// take the next vertex from the component
			int vertex = compVertices [pos ++];
			// add the appropriate edges to the result graph
			addRenumEdges (g, vertex, newNum, cur,
				srcVert, result);
		}
		++ cur;
	}
	// process the remaining vertices of the graph
	for (int vertex = 0; vertex < nVert; ++ vertex)
	{
		// debug
		if (newNum [vertex] > cur)
			throw "DEBUG: Wrong order.";
		// if the vertex has already been processed, skip it
		if (newNum [vertex] != cur)
			continue;
		// add the appropriate edges to the result graph
		result. addVertex ();
		addRenumEdges (g, vertex, newNum, cur, srcVert, result);
		++ cur;
	}

	// free memory and exit
	delete [] srcVert;
	if (!newNumbers)
		delete [] newNum;
	return 0;
} /* collapseVertices */

/// Computes the graph that represents connections between a number
/// of the first vertices in the given graph.
/// The vertices of the result graph are the first "nVert" vertices
/// from the source graph. An edge is added to the new graph
/// iff there exists a path from one vertex to another,
/// without going through any other vertex in that set.
/// Runs DFS starting from each of the first "nVert" vertices,
/// and thus may be a little inefficient in some cases.
template <class wType>
inline int connectionGraph (const diGraph<wType> &g, int nVert,
	diGraph<wType> &result)
{
	// remember the number of vertices in the input graph
	int nVertG = g. countVertices ();
	if (!nVertG)
		return 0;

	// prepare a bitfield in which visited vertices will be marked
	BitField visited;
	visited. allocate (nVertG);
	visited. clearall (nVertG);

	// run DFS for each of the starting vertices
	for (int startVertex = 0; startVertex < nVert; ++ startVertex)
	{
		// add this vertex to the resulting graph
		result. addVertex ();

		// prepare a counter and a stack of visited vertices
		int nVisited = 0;
		std::stack<int> s_visited;

		// prepare stacks for the DFS algorithm
		std::stack<int> s_vertex;
		std::stack<int> s_edge;

		// mark the starting vertex as visited
		visited. set (startVertex);
		s_visited. push (startVertex);
		++ nVisited;

		// use DFS to visit vertices reachable from that vertex
		int vertex = startVertex;
		int edge = 0;
		while (1)
		{
			// go back with the recursion
			// if all the edges have been processed
			if (edge >= g. countEdges (vertex))
			{
				// if this was the top recursion level
				// then quit
				if (s_vertex. empty ())
					break;

				// return from the recursion
				vertex = s_vertex. top ();
				s_vertex. pop ();
				edge = s_edge. top ();
				s_edge. pop ();
				continue;
			}

			// take the next vertex
			int next = g. getEdge (vertex, edge ++);

			// go to the deeper recursion level if not visited
			if (!visited. test (next))
			{
				// add an edge to the result if necessary
				if (next < nVert)
					result. addEdge (next);

				// store the previous variables at the stacks
				s_vertex. push (vertex);
				s_edge. push (edge);

				// take the new vertex
				vertex = next;
				edge = 0;

				// mark the new vertex as visited
				visited. set (vertex);
				s_visited. push (vertex);
				++ nVisited;
			}
		}

		// if this was the last vertex then skip the rest
		if (startVertex == nVert - 1)
			break;

		// mark each vertex as non-visited
		if (nVisited > (nVertG >> 6))
		{
			visited. clearall (nVertG);
		}
		else
		{
			while (!s_visited. empty ())
			{
				int vertex = s_visited. top ();
				s_visited. pop ();
				visited. clear (vertex);
			}
		}
	}

	// free memory and exit
	visited. free ();
	return 0;
} /* connectionGraph */

/// Computes the period of a strongly connected graph.
/// The period of a graph is defined as the greatest common divisor
/// of the lengths of all the cycles in the graph.
/// Period 1 means that the graph is aperiodic.
/// The complexity of this operation is linear (one DFS).
template <class wType>
inline int computePeriod (const diGraph<wType> &g)
{
	// remember the number of vertices in the input graph
	int nVertG = g. countVertices ();
	if (!nVertG)
		return 0;

	// prepare an array to record the depth of each visited vertex
	std::vector<int> visited (nVertG, 0);

	// run DFS starting at the first vertex
	// prepare stacks for the DFS algorithm
	std::stack<int> s_vertex;
	std::stack<int> s_edge;

	// mark the starting vertex as visited
	visited [0] = 1;

	// use DFS to visit vertices reachable from that vertex
	int vertex = 0;
	int edge = 0;
	int level = 1;
	int period = 0;
	while (1)
	{
		// go back with the recursion
		// if all the edges have been processed
		if (edge >= g. countEdges (vertex))
		{
			// if this was the top recursion level then quit
			if (s_vertex. empty ())
				break;

			// return from the recursion
			vertex = s_vertex. top ();
			s_vertex. pop ();
			edge = s_edge. top ();
			s_edge. pop ();
			-- level;
			continue;
		}

		// take the next vertex
		int next = g. getEdge (vertex, edge ++);

		// update the GCD of the cycle lengths
		if (visited [next])
		{
			// determine the period provided by the cycle
			int newPeriod = visited [next] - level - 1;
			if (newPeriod < 0)
				newPeriod = -newPeriod;

			// compute the GCD of the old and new periods
			int a = newPeriod;
			int b = period;
			while (b)
			{
				int t = b;
				b = a % b;
				a = t;
			}
			period = a;

			// if the graph turns out to be aperiodic
			// then immediately return the result
			if (period == 1)
				return period;
		}

		// go to the deeper recursion level if not visited
		else
		{
			// store the previous variables at the stacks
			s_vertex. push (vertex);
			s_edge. push (edge);

			// take the new vertex
			vertex = next;
			edge = 0;
			++ level;

			// mark the new vertex as visited
			visited [vertex] = level;
		}
	}

	// free memory and exit
	return period;
} /* computePeriod */

/// Computes the graph that represents flow-induced relations on Morse sets.
/// The vertices of the result graph are the first "nVert" vertices
/// from the source graph. An edge is added to the new graph
/// iff there exists a path from one vertex to another.
/// Edges that come from the transitive closure are not added.
template <class wType>
inline int inclusionGraph (const diGraph<wType> &g, int nVert,
	diGraph<wType> &result)
{
	// remember the number of vertices in the input graph
	int nVertG = g. countVertices ();
	if (!nVertG)
		return 0;

	// mark each vertex as non-visited
	BitField visited;
	visited. allocate (nVertG);
	visited. clearall (nVertG);

	// create the sets of vertices reachable from each vertex
	BitSets lists (nVertG, nVert);

	// prepare stacks for the DFS algorithm
	std::stack<int> s_vertex;
	std::stack<int> s_edge;

	// use DFS to propagate the reachable sets
	int startVertex = 0;
	int vertex = 0;
	int edge = 0;
	visited. set (vertex);
	while (1)
	{
		// go back with the recursion
		// if all the edges have been processed
		if (edge >= g. countEdges (vertex))
		{
			// if this was the top recursion level,
			// then find another starting point or quit
			if (s_vertex. empty ())
			{
				while ((startVertex < nVertG) &&
					(visited. test (startVertex)))
					++ startVertex;
				if (startVertex >= nVertG)
					break;
				vertex = startVertex;
				edge = 0;
				visited. set (vertex);
				continue;
			}

			// determine the previous vertex (to trace back to)
			int prev = s_vertex. top ();

			// add the list of the current vertex
			// to the list of the previous vertex
			// unless that was one of the first 'nVert' vertices
			if (vertex >= nVert)
				lists. addset (prev, vertex);

			// return from the recursion
			vertex = prev;
			s_vertex. pop ();
			edge = s_edge. top ();
			s_edge. pop ();
			continue;
		}

		// take the next vertex
		int next = g. getEdge (vertex, edge ++);

		// add the number to the list if appropriate
		if (next < nVert)
			lists. add (vertex, next);

		// go to the deeper recursion level if vertex not visited
		if (!visited. test (next))
		{
			// store the previous variables at the stacks
			s_vertex. push (vertex);
			s_edge. push (edge);

			// take the new vertex and mark as visited
			vertex = next;
			edge = 0;
			visited. set (vertex);
		}

		// add the list of the next vertex to the current one
		// if the vertex has already been visited before
		// and the vertex is not one of the first 'nVert' ones
		else if (next >= nVert)
		{
			lists. addset (vertex, next);
		}
	}

	// create the result graph based on the lists of edges
	for (int vertex = 0; vertex < nVert; ++ vertex)
	{
		result. addVertex ();
		int edge = lists. get (vertex);
		while (edge >= 0)
		{
			result. addEdge (edge);
			edge = lists. get (vertex, edge + 1);
		}
	}

	// free memory and exit
	visited. free ();
	return 0;
} /* inclusionGraph */

// --------------------------------------------------

/// A more complicated version of the 'inclusionGraph' function.
/// Computes the graph that represents flow-induced relations on Morse sets.
/// The vertices of the result graph are the first "nVert" vertices
/// from the source graph. An edge is added to the new graph
/// iff there exists a path from one vertex to another.
/// Edges that come from the transitive closure are not added.
/// Records vertices along connecting orbits using operator () with the
/// following arguments: source, target, vertex.
template<class wType, class TConn>
inline int inclusionGraph (const diGraph<wType> &g, int nVert,
	diGraph<wType> &result, TConn &connections)
{
	// remember the number of vertices in the input graph
	int nVertG = g. countVertices ();
	if (!nVertG)
		return 0;

	// mark each vertex as non-visited
	BitField visited;
	visited. allocate (nVertG);
	visited. clearall (nVertG);

	// create the sets of vertices reachable from each vertex
	BitSets forwardlists (nVertG, nVert);

	// prepare stacks for the DFS algorithm
	std::stack<int> s_vertex;
	std::stack<int> s_edge;

	// use DFS to propagate the reachable sets
	int startVertex = 0;
	int vertex = 0;
	int edge = 0;
	visited. set (vertex);
	while (1)
	{
		// go back with the recursion
		// if all the edges have been processed
		if (edge >= g. countEdges (vertex))
		{
			// if this was the top recursion level,
			// then find another starting point or quit
			if (s_vertex. empty ())
			{
				while ((startVertex < nVertG) &&
					(visited. test (startVertex)))
					++ startVertex;
				if (startVertex >= nVertG)
					break;
				vertex = startVertex;
				edge = 0;
				visited. set (vertex);
				continue;
			}

			// determine the previous vertex (to trace back to)
			int prev = s_vertex. top ();

			// add the list of the current vertex
			// to the list of the previous vertex
			// unless that was one of the first 'nVert' vertices
			if (vertex >= nVert)
				forwardlists. addset (prev, vertex);

			// return from the recursion
			vertex = prev;
			s_vertex. pop ();
			edge = s_edge. top ();
			s_edge. pop ();
			continue;
		}

		// take the next vertex
		int next = g. getEdge (vertex, edge ++);

		// add the number to the list if appropriate
		if (next < nVert)
			forwardlists. add (vertex, next);

		// go to the deeper recursion level if vertex not visited
		if (!visited. test (next))
		{
			// store the previous variables at the stacks
			s_vertex. push (vertex);
			s_edge. push (edge);

			// take the new vertex and mark as visited
			vertex = next;
			edge = 0;
			visited. set (vertex);
		}

		// add the list of the next vertex to the current one
		// if the vertex has already been visited before
		// and the vertex is not one of the first 'nVert' ones
		else if (next >= nVert)
		{
			forwardlists. addset (vertex, next);
		}
	}

	// ------------------------------------------

	// create the sets of vertices that can reach each vertex
	BitSets backlists (nVertG, nVert);

	diGraph<wType> gT;
	g. transpose (gT);

	// do a simple test for debugging purposes
	if (!s_vertex. empty () || !s_edge. empty ())
		throw "DEBUG: Nonempty stacks in 'inclusionGraph'.";

	// mark all the vertices as unvisited for the backward run
	visited. clearall (nVertG);

	// use DFS to propagate the reachable sets
	startVertex = 0;
	vertex = 0;
	edge = 0;
	visited. set (vertex);
	while (1)
	{
		// go back with the recursion
		// if all the edges have been processed
		if (edge >= gT. countEdges (vertex))
		{
			// if this was the top recursion level,
			// then find another starting point or quit
			if (s_vertex. empty ())
			{
				while ((startVertex < nVertG) &&
					(visited. test (startVertex)))
					++ startVertex;
				if (startVertex >= nVertG)
					break;
				vertex = startVertex;
				edge = 0;
				visited. set (vertex);
				continue;
			}

			// determine the previous vertex (to trace back to)
			int prev = s_vertex. top ();

			// add the list of the current vertex
			// to the list of the previous vertex
			// unless that was one of the first 'nVert' vertices
			if (vertex >= nVert)
				backlists. addset (prev, vertex);

			// return from the recursion
			vertex = prev;
			s_vertex. pop ();
			edge = s_edge. top ();
			s_edge. pop ();
			continue;
		}

		// take the next vertex
		int next = gT. getEdge (vertex, edge ++);

		// add the number to the list if appropriate
		if (next < nVert)
			backlists. add (vertex, next);

		// go to the deeper recursion level if vertex not visited
		if (!visited. test (next))
		{
			// store the previous variables at the stacks
			s_vertex. push (vertex);
			s_edge. push (edge);

			// take the new vertex and mark as visited
			vertex = next;
			edge = 0;
			visited. set (vertex);
		}

		// add the list of the next vertex to the current one
		// if the vertex has already been visited before
		// and the vertex is not one of the first 'nVert' ones
		else if (next >= nVert)
		{
			backlists. addset (vertex, next);
		}
	}

	// ------------------------------------------

	// record the connections to the obtained data structure
	for (int v = nVert; v < nVertG; ++ v)
	{
		int bvertex = backlists. get (v);
		while (bvertex >= 0)
		{
			int fvertex = forwardlists. get (v);
			while (fvertex >= 0)
			{
				connections (bvertex, fvertex, v);
				fvertex = forwardlists. get (v, fvertex + 1);
			}
			bvertex = backlists. get (v, bvertex + 1);
		}
	}

	// ------------------------------------------
	
	// create the result graph based on the lists of edges
	for (int vertex = 0; vertex < nVert; ++ vertex)
	{
		result. addVertex ();
		int edge = forwardlists. get (vertex);
		while (edge >= 0)
		{
			result. addEdge (edge);
			edge = forwardlists. get (vertex, edge + 1);
		}
	}

	// free memory and exit
	visited. free ();
	return 0;
} /* inclusionGraph */

// --------------------------------------------------

/// Creates the adjacency matrix of the given graph.
/// m [i] [j] is set to 1 if the graph g contains the edge i -> j,
/// using the assignment operator.
template <class wType, class matrix>
inline void graph2matrix (const diGraph<wType> &g, matrix &m)
{
	int nVert = g. countVertices ();
	for (int v = 0; v < nVert; ++ v)
	{
		int nEdges = g. countEdges (v);
		for (int e = 0; e < nEdges; ++ e)
		{
			int w = g. getEdge (v, e);
			m [v] [w] = 1;
		}
	}
	return;
} /* graph2matrix */

/// Creates a graph based on the given adjacency matrix.
/// If m [i] [j] is nonzero, then the edge i -> j is added to the graph.
/// It is assumed that the graph g is initially empty.
/// The size of the matrix (the number of vertices), n, must be given.
template <class wType, class matrix>
inline void matrix2graph (const matrix &m, int n, diGraph<wType> &g)
{
	for (int v = 0; v < n; ++ v)
	{
		g. addVertex ();
		for (int w = 0; w < n; ++ w)
			if (m [v] [w])
				g. addEdge (w);
	}
	return;
} /* matrix2graph */

// --------------------------------------------------

/// Computes the transitive closure of an acyclic graph
/// defined by its adjacency matrix, using the Warshall's algorithm:
/// S. Warshall, A theorem on Boolean matrices, J. ACM 9 (1962) 11-12.
template <class matrix>
inline void transitiveClosure (matrix &m, int n)
{
	for (int k = 0; k < n; ++ k)
	{
		for (int i = 0; i < n; ++ i)
		{
			for (int j = 0; j < n; ++ j)
			{
				if (m [i] [k] && m [k] [j])
					m [i] [j] = 1;
			}
		}
	}
	return;
} /* transitiveClosure */

/// Computes the transitive reduction of a CLOSED acyclic graph
/// defined by its adjacency matrix, using the algorithm by D. Gries,
/// A.J. Martin, J.L.A. van de Snepscheut and J.T. Udding, An algorithm
/// for transitive reduction of an acyclic graph, Science of Computer
/// Programming 12 (1989), 151-155.
/// WARNING: The input graph MUST BE CLOSED, use the "transitiveClosure"
/// algorithm first if this is not the case.
template <class matrix>
inline void transitiveReduction (matrix &m, int n)
{
	for (int k = n - 1; k >= 0; -- k)
	{
		for (int i = 0; i < n; ++ i)
		{
			for (int j = 0; j < n; ++ j)
			{
				if (m [i] [k] && m [k] [j])
					m [i] [j] = 0;
			}
		}
	}
	return;
} /* transitiveReduction */

/// Computes the transitive reduction of an arbitrary acyclic graph.
/// The output graph must be initially empty.
template <class wType>
inline void transitiveReduction (const diGraph<wType> &g,
	diGraph<wType> &gRed)
{
	int nVert = g. countVertices ();
	if (nVert <= 0)
		return;
	flatMatrix<char> m (nVert);
	m. clear (0);
	graph2matrix (g, m);
	transitiveClosure (m, nVert);
	transitiveReduction (m, nVert);
	matrix2graph (m, nVert, gRed);
	return;
} /* transitiveReduction */


} // namespace homology
} // namespace chomp

#endif // _CHOMP_STRUCT_DIGRAPH_H_

/// @}

